@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: ChunkStack.h 611 2006-12-20 04:12:46Z bengardner $
+ * $Id: ChunkStack.h 1158 2008-04-12 20:51:38Z bengardner $
  */
 
 #ifndef CHUNKSTACK_H_INCLUDED
@@ -20,7 +20,6 @@ public:
    {
       int     m_seqnum;
       chunk_t *m_pc;
-      chunk_t *m_trailer;  /* for AlignStack class */
    };
 
 protected:
@@ -56,10 +55,9 @@ public:
       return(m_len);
    }
 
-   void SetTopTrailer(chunk_t *pc);
-
    const Entry *Top() const;
    const Entry *Get(int idx) const;
+   chunk_t *GetChunk(int idx) const;
 
    chunk_t *Pop();
    void Push(chunk_t *pc, int seqnum);
