@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: align.cpp 1178 2008-05-09 20:46:04Z bengardner $
+ * $Id: align.cpp 1302 2008-07-18 22:49:53Z bengardner $
  */
 #include "uncrustify_types.h"
 #include "chunk_list.h"
@@ -19,10 +19,11 @@
 #include "unc_ctype.h"
 
 
-static chunk_t *align_var_def_brace(chunk_t *pc, int span);
+static chunk_t *align_var_def_brace(chunk_t *pc, int span, int *nl_count);
 chunk_t *align_trailing_comments(chunk_t *start);
 static void align_init_brace(chunk_t *start);
 static void align_func_params();
+static void align_same_func_call_params();
 static void align_func_proto(int span);
 static void align_oc_msg_spec(int span);
 
@@ -246,7 +247,7 @@ void align_all(void)
    if ((cpd.settings[UO_align_var_def_span].n > 0) ||
        (cpd.settings[UO_align_var_struct_span].n > 0))
    {
-      align_var_def_brace(chunk_get_head(), cpd.settings[UO_align_var_def_span].n);
+      align_var_def_brace(chunk_get_head(), cpd.settings[UO_align_var_def_span].n, NULL);
    }
 
    /* Align assignments */
@@ -279,6 +280,10 @@ void align_all(void)
       align_func_params();
    }
 
+   if (cpd.settings[UO_align_same_func_call_params].b)
+   {
+      align_same_func_call_params();
+   }
    /* Just in case something was aligned out of order... do it again */
    quick_align_again();
 }
@@ -331,6 +336,47 @@ void align_backslash_newline(void)
 void align_right_comments(void)
 {
    chunk_t *pc;
+   chunk_t *prev;
+   bool    skip;
+
+   for (pc = chunk_get_head(); pc != NULL; pc = chunk_get_next(pc))
+   {
+      if ((pc->type == CT_COMMENT) || (pc->type == CT_COMMENT_CPP))
+      {
+         skip = false;
+         if (pc->parent_type == CT_COMMENT_END)
+         {
+            prev = chunk_get_prev(pc);
+            if (pc->orig_col <= (prev->orig_col_end + cpd.settings[UO_align_right_cmt_gap].n))
+            {
+               LOG_FMT(LALTC, "NOT changing END comment on line %d\n",
+                       pc->orig_line);
+               skip = true;
+            }
+            if (!skip)
+            {
+               LOG_FMT(LALTC, "Changing END comment on line %d into a RIGHT-comment\n",
+                       pc->orig_line);
+               pc->flags |= PCF_RIGHT_COMMENT;
+            }
+         }
+
+         /* Change certain WHOLE comments into RIGHT-alignable comments */
+         if (pc->parent_type == CT_COMMENT_WHOLE)
+         {
+            int tmp_col = 1 + (pc->brace_level * cpd.settings[UO_indent_columns].n);
+
+            /* If the comment is further right than the brace level... */
+            if (pc->column > (tmp_col + cpd.settings[UO_align_right_cmt_gap].n))
+            {
+               LOG_FMT(LALTC, "Changing WHOLE comment on line %d into a RIGHT-comment\n",
+                       pc->orig_line);
+
+               pc->flags |= PCF_RIGHT_COMMENT;
+            }
+         }
+      }
+   }
 
    pc = chunk_get_head();
    while (pc != NULL)
@@ -675,6 +721,142 @@ static void align_func_params()
    }
 }
 
+static int align_params(chunk_t *start, chunk_t *chunks[], int num_chunks)
+{
+   int     count     = 0;
+   chunk_t *pc       = start;
+   bool    hit_comma = true;
+
+   pc = chunk_get_next_type(start, CT_FPAREN_OPEN, start->level);
+
+   while ((pc = chunk_get_next(pc)) != NULL)
+   {
+      if (chunk_is_newline(pc) ||
+          (count >= num_chunks) ||
+          (pc->type == CT_SEMICOLON) ||
+          ((pc->type == CT_FPAREN_CLOSE) && (pc->level == start->level)))
+      {
+         break;
+      }
+
+      if (pc->level == (start->level + 1))
+      {
+         if (hit_comma)
+         {
+            chunks[count++] = pc;
+            hit_comma   = false;
+         }
+         else if (pc->type == CT_COMMA)
+         {
+            hit_comma = true;
+         }
+      }
+   }
+   return(count);
+}
+
+static void align_same_func_call_params()
+{
+   chunk_t *pc;
+   chunk_t *align_root = NULL;
+   chunk_t *align_cur  = NULL;
+   int     align_len   = 0;
+   chunk_t    *chunks[16];
+   AlignStack as[16];
+   AlignStack fcn_as;
+   int        max_idx = 0;
+   int        cur_as;
+   int        idx;
+   const char *add_str = NULL;
+
+   fcn_as.Start(3);
+
+   for (pc = chunk_get_head(); pc != NULL; pc = chunk_get_next(pc))
+   {
+      if (pc->type != CT_FUNC_CALL)
+      {
+         if (chunk_is_newline(pc))
+         {
+            for (idx = 0; idx <= max_idx; idx++)
+            {
+               as[idx].NewLines(pc->nl_count);
+            }
+            fcn_as.NewLines(pc->nl_count);
+         }
+         continue;
+      }
+
+      fcn_as.Add(pc);
+      add_str = NULL;
+
+      if (align_root != NULL)
+      {
+         if ((pc->len == align_root->len) &&
+             (memcmp(pc->str, align_root->str, pc->len) == 0))
+         {
+            align_cur->align.next = pc;
+            align_cur             = pc;
+            align_len++;
+            add_str = "  Add";
+         }
+         else
+         {
+            LOG_FMT(LASFCP, "  ++ Ended with %d fcns\n", align_len);
+
+            /* Flush it all! */
+            fcn_as.Flush();
+            for (idx = 0; idx <= max_idx; idx++)
+            {
+               as[idx].Flush();
+            }
+            align_root = NULL;
+         }
+      }
+      if (align_root == NULL)
+      {
+         align_root = pc;
+         align_cur  = pc;
+         align_len  = 1;
+         add_str = "Start";
+      }
+
+      if (add_str != NULL)
+      {
+         LOG_FMT(LASFCP, "%s '%.*s' on line %d -",
+                 add_str, pc->len, pc->str, pc->orig_line);
+         cur_as = align_params(pc, chunks, ARRAY_SIZE(chunks));
+         LOG_FMT(LASFCP, " %d items:", cur_as);
+
+         for (idx = 0; idx < cur_as; idx++)
+         {
+            LOG_FMT(LASFCP, " [%.*s]", chunks[idx]->len, chunks[idx]->str);
+            if (idx > max_idx)
+            {
+               as[idx].Start(3);
+               if (!cpd.settings[UO_align_number_left].b)
+               {
+                  if ((chunks[idx]->type == CT_NUMBER_FP) ||
+                      (chunks[idx]->type == CT_NUMBER) ||
+                      (chunks[idx]->type == CT_POS) ||
+                      (chunks[idx]->type == CT_NEG))
+                  {
+                     as[idx].m_right_align = true;
+                  }
+               }
+               max_idx = idx;
+            }
+            as[idx].Add(chunks[idx]);
+         }
+         LOG_FMT(LASFCP, "\n");
+      }
+   }
+   fcn_as.End();
+   for (idx = 0; idx <= max_idx; idx++)
+   {
+      as[idx].End();
+   }
+}
+
 /**
  * Aligns all function prototypes in the file.
  */
@@ -725,7 +907,7 @@ static void align_func_proto(int span)
  * variable def align column.  Also aligns bit-colons, but that assumes that
  * bit-types are the same! But that should always be the case...
  */
-static chunk_t *align_var_def_brace(chunk_t *start, int span)
+static chunk_t *align_var_def_brace(chunk_t *start, int span, int *p_nl_count)
 {
    chunk_t    *pc;
    chunk_t    *next;
@@ -794,7 +976,7 @@ static chunk_t *align_var_def_brace(chunk_t *start, int span)
    as_br.m_gap = cpd.settings[UO_align_single_line_brace_gap].n;
 
    bool did_this_line = false;
-   pc = chunk_get_next_ncnl(start);
+   pc = chunk_get_next(start);
    while ((pc != NULL) && ((pc->level >= start->level) || (pc->level == 0)))
    {
       if (chunk_is_comment(pc))
@@ -834,14 +1016,28 @@ static chunk_t *align_var_def_brace(chunk_t *start, int span)
       /* process nested braces */
       if (pc->type == CT_BRACE_OPEN)
       {
-         pc = align_var_def_brace(pc, span);
+         int sub_nl_count = 0;
+
+         pc = align_var_def_brace(pc, span, &sub_nl_count);
+         if (sub_nl_count > 0)
+         {
+            fp_look_bro   = false;
+            did_this_line = false;
+            as.NewLines(sub_nl_count);
+            as_bc.NewLines(sub_nl_count);
+            as_br.NewLines(sub_nl_count);
+            if (p_nl_count != NULL)
+            {
+               *p_nl_count += sub_nl_count;
+            }
+         }
          continue;
       }
 
       /* Done with this brace set? */
       if (pc->type == CT_BRACE_CLOSE)
       {
-         pc = chunk_get_next_ncnl(pc);
+         pc = chunk_get_next(pc);
          break;
       }
 
@@ -852,6 +1048,10 @@ static chunk_t *align_var_def_brace(chunk_t *start, int span)
          as.NewLines(pc->nl_count);
          as_bc.NewLines(pc->nl_count);
          as_br.NewLines(pc->nl_count);
+         if (p_nl_count != NULL)
+         {
+            *p_nl_count += pc->nl_count;
+         }
       }
 
       /* don't align stuff inside parens/squares/angles */
@@ -978,9 +1178,12 @@ chunk_t *align_trailing_comments(chunk_t *start)
    int          nl_count = 0;
    ChunkStack   cs;
    CmtAlignType cmt_type_start, cmt_type_cur;
+   int          last_col, col;
 
    cmt_type_start = get_comment_align_type(pc);
 
+   last_col = pc->column;
+
    /* Find the max column */
    while ((pc != NULL) && (nl_count < cpd.settings[UO_align_right_cmt_span].n))
    {
@@ -990,6 +1193,13 @@ chunk_t *align_trailing_comments(chunk_t *start)
 
          if (cmt_type_cur == cmt_type_start)
          {
+            last_col = pc->column;
+
+            col = 1 + (pc->brace_level * cpd.settings[UO_indent_columns].n);
+            if (pc->column < col)
+            {
+               pc->column = col;
+            }
             align_add(cs, pc, max_col, 1, false);
             nl_count = 0;
          }
