@@ -7,7 +7,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: brace_cleanup.cpp 1176 2008-05-09 20:38:15Z bengardner $
+ * $Id: brace_cleanup.cpp 1267 2008-06-21 00:21:25Z bengardner $
  */
 
 #include "uncrustify_types.h"
@@ -604,6 +604,8 @@ static void parse_cleanup(struct parse_frame *frm, chunk_t *pc)
        (pc->type == CT_BRACE_OPEN) ||
        chunk_is_semicolon(pc) ||
        (pc->type == CT_COMMA) ||
+       (pc->type == CT_NOT) ||
+       (pc->type == CT_INV) ||
        (pc->type == CT_COLON) ||
        (pc->type == CT_QUESTION))
    {
@@ -662,11 +664,15 @@ static bool check_complex_statements(struct parse_frame *frm, chunk_t *pc)
    {
       if (pc->type == CT_IF)
       {
-         /* Replace CT_ELSE with CT_IF */
-         pc->type = CT_ELSEIF;
-         frm->pse[frm->pse_tos].type  = CT_ELSEIF;
-         frm->pse[frm->pse_tos].stage = BS_PAREN1;
-         return(true);
+         if (!cpd.settings[UO_indent_else_if].b ||
+             !chunk_is_newline(chunk_get_prev_nc(pc)))
+         {
+            /* Replace CT_ELSE with CT_IF */
+            pc->type = CT_ELSEIF;
+            frm->pse[frm->pse_tos].type  = CT_ELSEIF;
+            frm->pse[frm->pse_tos].stage = BS_PAREN1;
+            return(true);
+         }
       }
 
       /* Jump to the 'expecting brace' stage */
@@ -732,7 +738,7 @@ static bool check_complex_statements(struct parse_frame *frm, chunk_t *pc)
       pc->level       = frm->level;
       pc->brace_level = frm->brace_level;
 
-      /* Mark as a start of a statment */
+      /* Mark as a start of a statement */
       frm->stmt_count = 0;
       frm->expr_count = 0;
       pc->flags      |= PCF_STMT_START | PCF_EXPR_START;
