@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: chunk_list.h 972 2007-10-17 21:54:41Z bengardner $
+ * $Id: chunk_list.h 1030 2007-11-26 03:54:26Z bengardner $
  */
 
 #ifndef CHUNK_LIST_H_INCLUDED
@@ -28,6 +28,8 @@ chunk_t *chunk_get_next(chunk_t *cur);
 chunk_t *chunk_get_prev(chunk_t *cur);
 
 void chunk_swap(chunk_t *pc1, chunk_t *pc2);
+void chunk_swap_lines(chunk_t *pc1, chunk_t *pc2);
+chunk_t *chunk_first_on_line(chunk_t *pc);
 
 chunk_t *chunk_get_next_nl(chunk_t *cur);
 chunk_t *chunk_get_next_nc(chunk_t *cur);
@@ -213,11 +215,18 @@ bool chunk_same_preproc(chunk_t *pc1, chunk_t *pc2)
 
 /**
  * Returns true if it is safe to delete the newline token.
- * The prev and next chunks must have the same PCF_IN_PREPROC flag.
+ * The prev and next chunks must have the same PCF_IN_PREPROC flag AND
+ * the newline can't be after a C++ comment.
  */
 static_inline
 bool chunk_safe_to_del_nl(chunk_t *nl)
 {
+   chunk_t *tmp = chunk_get_prev(nl);
+
+   if ((tmp != NULL) && (tmp->type == CT_COMMENT_CPP))
+   {
+      return(false);
+   }
    return(chunk_same_preproc(chunk_get_prev(nl), chunk_get_next(nl)));
 }
 
