@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: combine.cpp 1183 2008-05-30 00:02:58Z bengardner $
+ * $Id: combine.cpp 1294 2008-07-18 01:04:49Z bengardner $
  */
 #include "uncrustify_types.h"
 #include "chunk_list.h"
@@ -212,7 +212,7 @@ void fix_symbols(void)
    {
       /* D stuff */
       if ((next->type == CT_PAREN_OPEN) &&
-          ((pc->type == CT_CAST) ||
+          ((pc->type == CT_D_CAST) ||
            (pc->type == CT_DELEGATE) ||
            (pc->type == CT_ALIGN)))
       {
@@ -220,7 +220,7 @@ void fix_symbols(void)
          tmp = set_paren_parent(next, pc->type);
 
          /* For a D cast - convert the next item */
-         if ((pc->type == CT_CAST) && (tmp != NULL))
+         if ((pc->type == CT_D_CAST) && (tmp != NULL))
          {
             if (tmp->type == CT_STAR)
             {
@@ -414,17 +414,42 @@ void fix_symbols(void)
          else if (pc->type == CT_TYPE)
          {
             /**
-             * If we are on a type, then we are either on a C++ style cast or
-             * we are on a function type.
+             * If we are on a type, then we are either on a C++ style cast, a
+             * function or we are on a function type.
              * The only way to tell for sure is to find the close paren and see
              * if it is followed by an open paren.
+             * "int(5.6)"
+             * "int()"
+             * "int(foo)(void)"
+             *
+             * FIXME: this check can be done better...
              */
             tmp = chunk_get_next_type(next, CT_PAREN_CLOSE, next->level);
             tmp = chunk_get_next(tmp);
             if ((tmp != NULL) && (tmp->type == CT_PAREN_OPEN))
             {
+               /* we have "TYPE(...)(" */
                pc->type = CT_FUNCTION;
             }
+            else
+            {
+               if ((pc->parent_type == CT_NONE) &&
+                   ((pc->flags & PCF_IN_TYPEDEF) == 0))
+               {
+                  tmp = chunk_get_next_ncnl(next);
+                  if ((tmp != NULL) && (tmp->type == CT_PAREN_CLOSE))
+                  {
+                     /* we have TYPE() */
+                     pc->type = CT_FUNCTION;
+                  }
+                  else
+                  {
+                     /* we have TYPE(...) */
+                     pc->type = CT_CPP_CAST;
+                     set_paren_parent(next, CT_CPP_CAST);
+                  }
+               }
+            }
          }
          else if (pc->type == CT_ATTRIBUTE)
          {
@@ -524,7 +549,7 @@ void fix_symbols(void)
        */
       if ((next != NULL) &&
           ((pc->flags & PCF_IN_TYPEDEF) == 0) &&
-          (pc->parent_type != CT_CAST) &&
+          (pc->parent_type != CT_CPP_CAST) &&
           ((pc->flags & PCF_IN_PREPROC) == 0) &&
           chunk_is_str(pc, ")", 1) &&
           chunk_is_str(next, "(", 1))
@@ -613,7 +638,7 @@ void fix_symbols(void)
 
       /* Detect a variable definition that starts with struct/enum/union */
       if (((pc->flags & PCF_IN_TYPEDEF) == 0) &&
-          (prev->parent_type != CT_CAST) &&
+          (prev->parent_type != CT_CPP_CAST) &&
           ((prev->flags & PCF_IN_FCN_DEF) == 0) &&
           ((pc->type == CT_STRUCT) ||
            (pc->type == CT_UNION) ||
@@ -766,24 +791,6 @@ void fix_symbols(void)
          pc = chunk_get_next_ncnl(pc);
       }
    }
-
-   /* 3rd pass - flag comments.
-    * Not done in first 2 loops because comments are skipped
-    */
-   for (pc = chunk_get_head(); pc != NULL; pc = chunk_get_next(pc))
-   {
-      if ((pc->type == CT_COMMENT) || (pc->type == CT_COMMENT_CPP))
-      {
-         prev = chunk_get_prev(pc);
-         next = chunk_get_next(pc);
-
-         if (!chunk_is_newline(prev) &&
-             ((next == NULL) || (next->type == CT_NEWLINE)))
-         {
-            pc->flags |= PCF_RIGHT_COMMENT;
-         }
-      }
-   }
 }
 
 /* Just hit an assign. Go backwards until we hit an open brace/paren/square or
@@ -961,6 +968,11 @@ static chunk_t *process_return(chunk_t *pc)
       return(next);
    }
 
+   if (cpd.settings[UO_nl_return_expr].a != AV_IGNORE)
+   {
+      newline_iarf(pc, cpd.settings[UO_nl_return_expr].a);
+   }
+
    if (next->type == CT_PAREN_OPEN)
    {
       /* See if the return is fully paren'd */
@@ -1028,7 +1040,7 @@ static chunk_t *process_return(chunk_t *pc)
       chunk.orig_line   = pc->orig_line;
       chunk.parent_type = CT_RETURN;
       chunk.flags       = pc->flags & PCF_COPY_FLAGS;
-      chunk_add_after(&chunk, pc);
+      chunk_add_before(&chunk, next);
 
       chunk.type      = CT_PAREN_CLOSE;
       chunk.str       = ")";
@@ -1069,6 +1081,7 @@ static bool is_ucase_str(const char *str, int len)
 static void fix_casts(chunk_t *start)
 {
    chunk_t    *pc;
+   chunk_t    *prev;
    chunk_t    *first;
    chunk_t    *after;
    chunk_t    *last = NULL;
@@ -1084,6 +1097,13 @@ static void fix_casts(chunk_t *start)
 
    LOG_FMT(LCASTS, "%s:line %d, col %d:", __func__, start->orig_line, start->orig_col);
 
+   prev = chunk_get_prev_ncnl(start);
+   if ((prev != NULL) && (prev->type == CT_PP_DEFINED))
+   {
+      LOG_FMT(LCASTS, " -- not a cast - after defined\n");
+      return;
+   }
+
    /* Make sure there is only WORD, TYPE, and '*' before the close paren */
    pc    = chunk_get_next_ncnl(start);
    first = pc;
@@ -1241,14 +1261,14 @@ static void fix_casts(chunk_t *start)
       }
    }
 
-   start->parent_type       = CT_CAST;
-   paren_close->parent_type = CT_CAST;
+   start->parent_type       = CT_C_CAST;
+   paren_close->parent_type = CT_C_CAST;
 
-   LOG_FMT(LCASTS, " -- %s cast: (", verb);
+   LOG_FMT(LCASTS, " -- %s c-cast: (", verb);
 
    for (pc = first; pc != paren_close; pc = chunk_get_next_ncnl(pc))
    {
-      pc->parent_type = CT_CAST;
+      pc->parent_type = CT_C_CAST;
       make_type(pc);
       LOG_FMT(LCASTS, " %.*s", pc->len, pc->str);
    }
@@ -1313,7 +1333,7 @@ static void fix_enum_struct_union(chunk_t *pc)
    int     in_fcn_paren = pc->flags & PCF_IN_FCN_DEF;
 
    /* Make sure this wasn't a cast */
-   if (pc->parent_type == CT_CAST)
+   if (pc->parent_type == CT_C_CAST)
    {
       return;
    }
@@ -1578,7 +1598,7 @@ void combine_labels(void)
          }
          if (question_count > 0)
          {
-            next->type = CT_Q_COLON;
+            next->type = CT_COND_COLON;
             question_count--;
          }
          else if (hit_case)
@@ -2097,19 +2117,60 @@ static void mark_function(chunk_t *pc)
    chunk_t *semi = NULL;
    chunk_t *paren_open;
    chunk_t *paren_close;
+   chunk_t *pc_op = NULL;
 
    prev = chunk_get_prev_ncnlnp(pc);
    next = chunk_get_next_ncnlnp(pc);
 
+   /* Find out what is before the operator */
+   if (pc->parent_type == CT_OPERATOR)
+   {
+      pc_op = chunk_get_prev_type(pc, CT_OPERATOR, pc->level);
+      if ((pc_op != NULL) && (pc_op->flags & PCF_EXPR_START))
+      {
+         pc->type = CT_FUNC_CALL;
+      }
+      if (cpd.lang_flags & LANG_CPP)
+      {
+         tmp = pc;
+         while ((tmp = chunk_get_prev_ncnl(tmp)) != NULL)
+         {
+            if (tmp->type == CT_BRACE_CLOSE)
+            {
+               break;
+            }
+            if (tmp->type == CT_TEMPLATE)
+            {
+               pc->type = CT_FUNC_DEF;
+               break;
+            }
+            if (tmp->type == CT_BRACE_OPEN)
+            {
+               if (tmp->parent_type == CT_FUNC_DEF)
+               {
+                  pc->type = CT_FUNC_CALL;
+               }
+               if ((tmp->parent_type == CT_CLASS) ||
+                   (tmp->parent_type == CT_STRUCT))
+               {
+                  pc->type = CT_FUNC_DEF;
+               }
+               break;
+            }
+         }
+      }
+   }
+
    if (chunk_is_star(next) || chunk_is_addr(next))
    {
       next = chunk_get_next_ncnlnp(next);
    }
 
-   LOG_FMT(LFCN, "%s: %d] %.*s[%s] - level=%d/%d, next=%.*s[%s] - level=%d\n",
+   LOG_FMT(LFCN, "%s: %d] %.*s[%s] - parent=%s level=%d/%d, next=%.*s[%s] - level=%d\n",
            __func__,
            pc->orig_line, pc->len, pc->str,
-           get_token_name(pc->type), pc->level, pc->brace_level,
+           get_token_name(pc->type), get_token_name(pc->parent_type),
+           pc->level, pc->brace_level,
            next->len, next->str, get_token_name(next->type), next->level);
 
    if (pc->flags & PCF_IN_CONST_ARGS)
@@ -2323,6 +2384,7 @@ static void mark_function(chunk_t *pc)
          if ((prev->type != CT_OPERATOR) &&
              (prev->type != CT_TSQUARE) &&
              (prev->type != CT_ANGLE_CLOSE) &&
+             (prev->type != CT_QUALIFIER) &&
              (prev->type != CT_TYPE) &&
              (prev->type != CT_WORD) &&
              !chunk_is_addr(prev) &&
@@ -2871,7 +2933,9 @@ static void mark_define_expressions(void)
    {
       if (!in_define)
       {
-         if (pc->type == CT_PP_DEFINE)
+         if ((pc->type == CT_PP_DEFINE) ||
+             (pc->type == CT_PP_IF) ||
+             (pc->type == CT_PP_ELSE))
          {
             in_define = true;
             first     = true;
