@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: defines.cpp 971 2007-10-17 21:26:22Z bengardner $
+ * $Id: defines.cpp 1088 2008-01-30 01:24:42Z bengardner $
  */
 
 #include "uncrustify_types.h"
@@ -14,7 +14,7 @@
 #include <cstring>
 #include <cerrno>
 #include <cstdlib>
-#include <cctype>
+#include "unc_ctype.h"
 
 typedef struct
 {
