@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: indent.cpp 1180 2008-05-09 21:03:39Z bengardner $
+ * $Id: indent.cpp 1297 2008-07-18 01:32:15Z bengardner $
  */
 #include "uncrustify_types.h"
 #include "chunk_list.h"
@@ -574,7 +574,6 @@ void indent_text(void)
                  (pc->type == CT_COMMA) ||
                  (pc->type == CT_BRACE_OPEN) ||
                  (pc->type == CT_SPAREN_CLOSE) ||
-                 (pc->type == CT_FPAREN_CLOSE) ||
                  ((pc->type == CT_SQUARE_OPEN) && (pc->parent_type == CT_ASSIGN))))
             {
                indent_pse_pop(frm, pc);
@@ -949,11 +948,13 @@ void indent_text(void)
                (pc->type == CT_SQUARE_OPEN) ||
                (pc->type == CT_ANGLE_OPEN))
       {
-         /* Open parens and squares - never update indent_column */
+         /* Open parens and squares - never update indent_column, unless right
+          * after a newline.
+          */
          indent_pse_push(frm, pc);
          frm.pse[frm.pse_tos].indent = pc->column + pc->len;
 
-         if ((pc->type == CT_FPAREN_OPEN) &&
+         if (((pc->type == CT_FPAREN_OPEN) || (pc->type == CT_ANGLE_OPEN)) &&
              ((cpd.settings[UO_indent_func_call_param].b &&
                (pc->parent_type == CT_FUNC_CALL))
               ||
@@ -964,6 +965,9 @@ void indent_text(void)
               (cpd.settings[UO_indent_func_class_param].b &&
                (pc->parent_type == CT_FUNC_CLASS))
               ||
+              (cpd.settings[UO_indent_template_param].b &&
+               (pc->parent_type == CT_TEMPLATE))
+              ||
               (cpd.settings[UO_indent_func_ctor_var_param].b &&
                (pc->parent_type == CT_FUNC_CTOR_VAR))
               ||
@@ -994,6 +998,7 @@ void indent_text(void)
          }
 
          else if ((chunk_is_str(pc, "(", 1) && !cpd.settings[UO_indent_paren_nl].b) ||
+                  (chunk_is_str(pc, "<", 1) && !cpd.settings[UO_indent_paren_nl].b) || /* TODO: add indent_angle_nl? */
                   (chunk_is_str(pc, "[", 1) && !cpd.settings[UO_indent_square_nl].b))
          {
             next = chunk_get_next_nc(pc);
@@ -1015,6 +1020,13 @@ void indent_text(void)
             }
          }
 
+         if ((pc->type == CT_FPAREN_OPEN) &&
+             chunk_is_newline(chunk_get_prev(pc)) &&
+             !chunk_is_newline(chunk_get_next(pc)))
+         {
+            frm.pse[frm.pse_tos].indent = frm.pse[frm.pse_tos].indent + indent_size;
+            indent_column_set(frm.pse[frm.pse_tos].indent);
+         }
          frm.pse[frm.pse_tos].indent_tmp = frm.pse[frm.pse_tos].indent;
          frm.paren_count++;
       }
@@ -1052,7 +1064,14 @@ void indent_text(void)
          if (pc->level == pc->brace_level)
          {
             indent_pse_push(frm, pc);
-            frm.pse[frm.pse_tos].indent     = frm.pse[frm.pse_tos - 1].indent + pc->len + 1;
+            if (chunk_is_newline(chunk_get_next(pc)))
+            {
+               frm.pse[frm.pse_tos].indent = frm.pse[frm.pse_tos - 1].indent + indent_size;
+            }
+            else
+            {
+               frm.pse[frm.pse_tos].indent = frm.pse[frm.pse_tos - 1].indent + pc->len + 1;
+            }
             frm.pse[frm.pse_tos].indent_tmp = frm.pse[frm.pse_tos - 1].indent;
          }
       }
@@ -1158,7 +1177,7 @@ void indent_text(void)
                     __func__, pc->orig_line, indent_column, pc->len, pc->str);
             reindent_line(pc, indent_column);
          }
-         else if (chunk_is_paren_close(pc))
+         else if (chunk_is_paren_close(pc) || (pc->type == CT_ANGLE_CLOSE))
          {
             /* This is a big hack. We assume that since we hit a paren close,
              * that we just removed a paren open */
@@ -1542,7 +1561,7 @@ void indent_preproc(void)
       /* Adjust the indent of the '#' */
       if ((cpd.settings[UO_pp_indent].a & AV_ADD) != 0)
       {
-         reindent_line(pc, 1 + pp_level);
+         reindent_line(pc, 1 + pp_level * cpd.settings[UO_pp_indent_count].n);
       }
       else if ((cpd.settings[UO_pp_indent].a & AV_REMOVE) != 0)
       {
