@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: keywords.cpp 1169 2008-05-02 01:19:53Z bengardner $
+ * $Id: keywords.cpp 1246 2008-06-15 22:30:09Z bengardner $
  */
 
 #include "uncrustify_types.h"
@@ -69,7 +69,7 @@ static const chunk_tag_t keywords[] =
    { "byte",             CT_TYPE,         LANG_CS | LANG_D | LANG_JAVA                                    },
    { "callback",         CT_QUALIFIER,    LANG_VALA                                                       },
    { "case",             CT_CASE,         LANG_ALL                                                        }, // PAWN
-   { "cast",             CT_CAST,         LANG_D                                                          },
+   { "cast",             CT_D_CAST,       LANG_D                                                          },
    { "catch",            CT_CATCH,        LANG_CPP | LANG_CS | LANG_D | LANG_JAVA                         },
    { "cdouble",          CT_TYPE,         LANG_D                                                          },
    { "cent",             CT_TYPE,         LANG_D                                                          },
