@@ -9,14 +9,14 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: logger.cpp 824 2007-07-27 20:41:59Z bengardner $
+ * $Id: logger.cpp 1106 2008-02-21 03:12:46Z bengardner $
  */
 
 #include "logger.h"
 
 #include <cstdio>
 #include <stdarg.h>
-#include <cctype>
+#include "unc_ctype.h"
 
 
 /** Private log structure */
@@ -133,8 +133,9 @@ static void log_flush(bool force_nl)
  * Starts the log statment by flushing if needed and printing the header
  *
  * @param sev  The log severity
+ * @return     The number of bytes available
  */
-static void log_start(log_sev_t sev)
+static size_t log_start(log_sev_t sev)
 {
    if (sev != g_log.sev)
    {
@@ -146,16 +147,15 @@ static void log_start(log_sev_t sev)
       g_log.in_log = false;
    }
 
-   if (!g_log.in_log)
+   /* If not in a log, the buffer is empty. Add the header, if enabled. */
+   if (!g_log.in_log && g_log.show_hdr)
    {
-      /* Add the header, if enabled */
-      if (g_log.show_hdr)
-      {
-         g_log.buf_len += snprintf(&g_log.buf[g_log.buf_len],
-                                   sizeof(g_log.buf) - g_log.buf_len,
-                                   "<%d>", sev);
-      }
+      g_log.buf_len = snprintf(g_log.buf, sizeof(g_log.buf), "<%d>", sev);
    }
+
+   int cap = ((int)sizeof(g_log.buf) - 2) - g_log.buf_len;
+
+   return((cap > 0) ? (size_t)cap : 0);
 }
 
 /**
@@ -185,15 +185,18 @@ void log_str(log_sev_t sev, const char *str, int len)
       return;
    }
 
-   if ((g_log.buf_len + len) < (int)sizeof(g_log.buf))
+   size_t cap = log_start(sev);
+   if (cap > 0)
    {
-      log_start(sev);
-
-      memcpy(&g_log.buf[g_log.buf_len], str, len + 1);
-      g_log.buf_len += len;
-
-      log_end();
+      if (len > (int)cap)
+      {
+         len = cap;
+      }
+      memcpy(&g_log.buf[g_log.buf_len], str, len);
+      g_log.buf_len           += len;
+      g_log.buf[g_log.buf_len] = 0;
    }
+   log_end();
 }
 
 /**
@@ -206,21 +209,35 @@ void log_str(log_sev_t sev, const char *str, int len)
 void log_fmt(log_sev_t sev, const char *fmt, ...)
 {
    va_list args;
+   int     len;
+   size_t  cap;
 
    if ((fmt == NULL) || !log_sev_on(sev))
    {
       return;
    }
 
-   log_start(sev);
+   /* Some implementation of vsnprintf() return the number of characters
+    * that would have been stored if the buffer was large enough instead of
+    * the number of characters actually stored.
+    */
+   cap = log_start(sev);
 
    /* Add on the variable log parameters to the log string */
    va_start(args, fmt);
-   g_log.buf_len += vsnprintf(&g_log.buf[g_log.buf_len],
-                              sizeof(g_log.buf) - g_log.buf_len,
-                              fmt, args);
+   len = vsnprintf(&g_log.buf[g_log.buf_len], cap, fmt, args);
    va_end(args);
 
+   if (len > 0)
+   {
+      if (len > (int)cap)
+      {
+         len = cap;
+      }
+      g_log.buf_len += len;
+      g_log.buf[g_log.buf_len] = 0;
+   }
+
    log_end();
 }
 
@@ -314,7 +331,7 @@ void log_hex_blk(log_sev_t sev, const void *data, int len)
       buf[str_idx + 1] = to_hex_char(tmp);
       str_idx         += 3;
 
-      buf[chr_idx++] = isprint(tmp) ? tmp : '.';
+      buf[chr_idx++] = unc_isprint(tmp) ? tmp : '.';
 
       total++;
       count++;
