@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: newlines.cpp 1171 2008-05-02 01:57:37Z bengardner $
+ * $Id: newlines.cpp 1294 2008-07-18 01:04:49Z bengardner $
  */
 #include "uncrustify_types.h"
 #include "chunk_list.h"
@@ -22,6 +22,35 @@ static bool one_liner_nl_ok(chunk_t *pc);
 static void nl_handle_define(chunk_t *pc);
 
 
+/**
+ * Check to see if we are allowed to double the newline.
+ * The only case where we can't is if nl_squeeze_ifdef=true and a preproc is
+ * after the next newline.
+ */
+static void double_newline(chunk_t *nl)
+{
+   chunk_t *prev = chunk_get_prev(nl);
+
+   LOG_FMT(LNEWLINE, "%s: add newline after %.*s on line %d",
+           __func__, prev->len, prev->str, prev->orig_line);
+
+   if (cpd.settings[UO_nl_squeeze_ifdef].b)
+   {
+      chunk_t *pc = chunk_get_next(nl);
+
+      LOG_FMT(LNEWLINE, " [%s]", get_token_name(pc->type));
+
+      if ((pc != NULL) && (pc->type == CT_PREPROC) &&
+          (pc->parent_type == CT_PP_ENDIF))
+      {
+         LOG_FMT(LNEWLINE, " - denied\n");
+         return;
+      }
+   }
+   LOG_FMT(LNEWLINE, " - done\n");
+   nl->nl_count = 2;
+}
+
 /*
  * Basic approach:
  * 1. Find next open brace
@@ -144,11 +173,6 @@ static void newline_min_after2(chunk_t *ref, INT32 count,
    }
 }
 
-/**
- * Adds a newline between the two chunks if there isn't one already.
- */
-#define newline_add_between(start, end)     newline_add_between2(start, end, __func__, __LINE__)
-
 /**
  * Add a newline between two tokens.
  * If there is already a newline between then, nothing is done.
@@ -164,8 +188,8 @@ static void newline_min_after2(chunk_t *ref, INT32 count,
  *    if (...)   //comment
  *    {
  */
-static chunk_t *newline_add_between2(chunk_t *start, chunk_t *end,
-                                     const char *func, int line)
+chunk_t *newline_add_between2(chunk_t *start, chunk_t *end,
+                              const char *func, int line)
 {
    chunk_t *pc;
 
@@ -225,11 +249,8 @@ static chunk_t *newline_add_between2(chunk_t *start, chunk_t *end,
  * @param end     The ending chunk (cannot be a newline)
  * @return        true/false - removed something
  */
-#define newline_del_between(start, end) \
-   newline_del_between2(start, end, __func__, __LINE__)
-
-static void newline_del_between2(chunk_t *start, chunk_t *end,
-                                 const char *func, int line)
+void newline_del_between2(chunk_t *start, chunk_t *end,
+                          const char *func, int line)
 {
    chunk_t *next;
    chunk_t *prev;
@@ -420,7 +441,7 @@ static void newlines_if_for_while_switch_pre_blank_lines(chunk_t *start, argval_
             {
                if (last_nl->nl_count < 2)
                {
-                  last_nl->nl_count = 2;
+                  double_newline(last_nl);
                }
             }
             else
@@ -433,7 +454,7 @@ static void newlines_if_for_while_switch_pre_blank_lines(chunk_t *start, argval_
                }
                if ((last_nl = newline_add_after(pc)) != NULL)
                {
-                  last_nl->nl_count = 2;
+                  double_newline(last_nl);
                }
             }
          }
@@ -589,10 +610,10 @@ static void newlines_if_for_while_switch_post_blank_lines(chunk_t *start, argval
          {
             nl_count += next->nl_count;
          }
+
          /* if we have no newlines, add one and make it double */
          if (nl_count == 0)
          {
-
             if (((next = chunk_get_next(pc)) != NULL) &&
                 chunk_is_comment(next))
             {
@@ -603,8 +624,7 @@ static void newlines_if_for_while_switch_post_blank_lines(chunk_t *start, argval
             {
                return;
             }
-
-            next->nl_count = 2;
+            double_newline(next);
          }
          else if (nl_count == 1) /* if we don't have enough newlines */
          {
@@ -615,8 +635,24 @@ static void newlines_if_for_while_switch_post_blank_lines(chunk_t *start, argval
             }
             else
             {
-               /* make nl after double */
-               next->nl_count = 2;
+               prev = chunk_get_prev_nnl(next);
+               pc   = chunk_get_next_nl(next);
+               LOG_FMT(LSYS, "  -- pc1=%.*s [%s]\n", pc->len, pc->str, get_token_name(pc->type));
+
+               pc   = chunk_get_next(pc);
+               LOG_FMT(LSYS, "  -- pc2=%.*s [%s]\n", pc->len, pc->str, get_token_name(pc->type));
+               if ((pc != NULL) && (pc->type == CT_PREPROC) &&
+                   (pc->parent_type == CT_PP_ENDIF) &&
+                   cpd.settings[UO_nl_squeeze_ifdef].b)
+               {
+                  LOG_FMT(LNEWLINE, "%s: cannot add newline after line %d due to nl_squeeze_ifdef\n",
+                          __func__, prev->orig_line);
+               }
+               else
+               {
+                  /* make nl after double */
+                  double_newline(next);
+               }
             }
          }
       }
@@ -1015,9 +1051,7 @@ static void newline_case(chunk_t *start)
    {
       if (chunk_is_newline(pc) && (pc->nl_count < 2))
       {
-         //         fprintf(stderr, "%s: newline before line %d\n",
-         //                 __func__, start->orig_line);
-         pc->nl_count = 2;
+         double_newline(pc);
       }
    }
 }
@@ -1067,7 +1101,7 @@ static void newline_return(chunk_t *start)
       {
          if (pc->nl_count < 2)
          {
-            pc->nl_count = 2;
+            double_newline(pc);
          }
          return;
       }
@@ -1080,7 +1114,7 @@ static void newline_return(chunk_t *start)
  * @param pc   The chunk
  * @param av   The IARF value
  */
-static void newline_iarf(chunk_t *pc, argval_t av)
+void newline_iarf(chunk_t *pc, argval_t av)
 {
    chunk_t *next;
 
@@ -1322,6 +1356,18 @@ void newlines_cleanup_braces(void)
       else if (pc->type == CT_ELSE)
       {
          newlines_cuddle_uncuddle(pc, cpd.settings[UO_nl_brace_else].a);
+         next = chunk_get_next_ncnl(pc);
+         if ((next != NULL) && (next->type == CT_ELSEIF))
+         {
+            if (cpd.settings[UO_nl_else_if].a & AV_REMOVE)
+            {
+               newline_del_between(pc, next);
+            }
+            if (cpd.settings[UO_nl_else_if].a & AV_ADD)
+            {
+               newline_add_between(pc, next);
+            }
+         }
          newlines_do_else(pc, cpd.settings[UO_nl_else_brace].a);
       }
       else if (pc->type == CT_TRY)
@@ -1432,7 +1478,7 @@ void newlines_cleanup_braces(void)
                }
                if (prev->nl_count < 2)
                {
-                  prev->nl_count = 2;
+                  double_newline(prev);
                }
             }
          }
@@ -1564,10 +1610,14 @@ void newlines_cleanup_braces(void)
          if (pc->parent_type == CT_TEMPLATE)
          {
             next = chunk_get_next_ncnl(pc);
-            if ((next != NULL) && (next->type == CT_CLASS) &&
+            if ((next != NULL) &&
                 (next->level == next->brace_level))
             {
-               newline_iarf(pc, cpd.settings[UO_nl_template_class].a);
+               tmp = chunk_get_prev_ncnl(chunk_get_prev_type(pc, CT_ANGLE_OPEN, pc->level));
+               if ((tmp != NULL) && (tmp->type == CT_TEMPLATE))
+               {
+                  newline_iarf(pc, cpd.settings[UO_nl_template_class].a);
+               }
             }
          }
       }
@@ -1743,14 +1793,14 @@ void newlines_squeeze_ifdef(void)
    chunk_t *ppr;
    chunk_t *pnl;
    chunk_t *nnl;
+   chunk_t *tmp1;
+   chunk_t *tmp2;
 
    for (pc = chunk_get_head(); pc != NULL; pc = chunk_get_next_ncnl(pc))
    {
       if ((pc->type == CT_PREPROC) && (pc->level > 0))
       {
          ppr = chunk_get_next(pc);
-         //          fprintf(stderr, "%s: %s online %d\n",
-         //                  __func__, get_token_name(ppr->type), ppr->orig_line);
 
          if ((ppr->type == CT_PP_IF) ||
              (ppr->type == CT_PP_ELSE) ||
@@ -1765,23 +1815,27 @@ void newlines_squeeze_ifdef(void)
 
             if (nnl != NULL)
             {
-               //                fprintf(stderr, "%s: next nl on line %d, count=%d\n",
-               //                        __func__, nnl->orig_line, nnl->nl_count);
-
                if (pnl != NULL)
                {
-                  //                  fprintf(stderr, "%s: prev nl on line %d, count=%d\n",
-                  //                          __func__, pnl->orig_line, pnl->nl_count);
                   if (pnl->nl_count > 1)
                   {
                      nnl->nl_count += pnl->nl_count - 1;
                      pnl->nl_count  = 1;
+
+                     tmp1 = chunk_get_prev_nnl(pnl);
+                     tmp2 = chunk_get_prev_nnl(nnl);
+
+                     LOG_FMT(LNEWLINE, "%s: moved from after line %d to after %d\n",
+                             __func__, tmp1->orig_line, tmp2->orig_line);
                   }
                }
                else
                {
                   if (nnl->nl_count > 1)
                   {
+                     tmp1 = chunk_get_prev_nnl(nnl);
+                     LOG_FMT(LNEWLINE, "%s: trimmed newlines after line %d from %d\n",
+                             __func__, tmp1->orig_line, nnl->nl_count);
                      nnl->nl_count = 1;
                   }
                }
@@ -2221,7 +2275,7 @@ static void newlines_double_space_struct_enum_union(chunk_t *open_brace)
       {
          if (pc->nl_count < 2)
          {
-            pc->nl_count = 2;
+            double_newline(pc);
          }
       }
    }
