@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: options.cpp 1192 2008-06-06 19:12:54Z bengardner $
+ * $Id: options.cpp 1300 2008-07-18 22:17:50Z bengardner $
  */
 #include "uncrustify_types.h"
 #include "args.h"
@@ -167,8 +167,13 @@ void register_options(void)
                   "Add or remove space after reference sign '&', if followed by a word.");
    unc_add_option("sp_after_type", UO_sp_after_type, AT_IARF,
                   "Add or remove space between type and word");
+   unc_add_option("sp_template_angle", UO_sp_template_angle, AT_IARF,
+                  "Add or remove space in 'template <' vs 'template<'.\n"
+                  "If set to ignore, sp_before_angle is used.");
    unc_add_option("sp_before_angle", UO_sp_before_angle, AT_IARF,
                   "Add or remove space before '<>'");
+   unc_add_option("sp_inside_angle", UO_sp_inside_angle, AT_IARF,
+                  "Add or remove space inside '<' and '>'");
    unc_add_option("sp_after_angle", UO_sp_after_angle, AT_IARF,
                   "Add or remove space after '<>'");
    unc_add_option("sp_angle_paren", UO_sp_angle_paren, AT_IARF,
@@ -209,8 +214,10 @@ void register_options(void)
                   "Add or remove space before class ':'");
    unc_add_option("sp_after_operator", UO_sp_after_operator, AT_IARF,
                   "Add or remove space between 'operator' and operator sign");
+   unc_add_option("sp_after_operator_sym", UO_sp_after_operator_sym, AT_IARF,
+                  "Add or remove space between the operator symbol and the open paren, as in 'operator ++('");
    unc_add_option("sp_after_cast", UO_sp_after_cast, AT_IARF,
-                  "Add or remove space after cast");
+                  "Add or remove space after C/D cast, ie 'cast(int)a' vs 'cast(int) a' or '(int)a' vs '(int) a'");
    unc_add_option("sp_inside_paren_cast", UO_sp_inside_paren_cast, AT_IARF,
                   "Add or remove spaces inside cast parens");
    unc_add_option("sp_sizeof_paren", UO_sp_sizeof_paren, AT_IARF,
@@ -225,8 +232,6 @@ void register_options(void)
                   "Add or remove space inside '{' and '}'");
    unc_add_option("sp_inside_braces_empty", UO_sp_inside_braces_empty, AT_IARF,
                   "Add or remove space inside '{}'");
-   unc_add_option("sp_inside_angle", UO_sp_inside_angle, AT_IARF,
-                  "Add or remove space inside '<' and '>'");
    unc_add_option("sp_type_func", UO_sp_type_func, AT_IARF,
                   "Add or remove space between return type and function name\n"
                   "A minimum of 1 is forced except for pointer return types.");
@@ -250,6 +255,8 @@ void register_options(void)
                   "Add or remove space between 'return' and '('");
    unc_add_option("sp_attribute_paren", UO_sp_attribute_paren, AT_IARF,
                   "Add or remove space between '__attribute__' and '('");
+   unc_add_option("sp_defined_paren", UO_sp_defined_paren, AT_IARF,
+                  "Add or remove space between 'defined' and '(' in '#if defined (FOO)'");
    unc_add_option("sp_macro", UO_sp_macro, AT_IARF,
                   "Add or remove space between macro and value");
    unc_add_option("sp_macro_func", UO_sp_macro_func, AT_IARF,
@@ -292,6 +299,9 @@ void register_options(void)
    unc_add_option("sp_incdec", UO_sp_incdec, AT_IARF,
                   "Add or remove space before or after '++' and '--', as in '(--x)' or 'y++;'");
 
+   unc_add_option("sp_before_nl_cont", UO_sp_before_nl_cont, AT_IARF,
+                  "Add or remove space before a backslash-newline at the end of a line");
+
    unc_add_option("sp_after_oc_scope", UO_sp_after_oc_scope, AT_IARF,
                   "Add or remove space after the scope '+' or '-', as in '-(void) foo;' or '+(int) bar;'");
    unc_add_option("sp_before_oc_colon", UO_sp_before_oc_colon, AT_IARF,
@@ -301,6 +311,12 @@ void register_options(void)
                   "Add or remove space after the (type) in message specs\n"
                   "'-(int) f: (int) x;' vs '+(int)f : (int)x;'");
 
+   unc_add_option("sp_cond_colon", UO_sp_cond_colon, AT_IARF,
+                  "Add or remove space around the ':' in 'b ? t : f'");
+   unc_add_option("sp_cond_question", UO_sp_cond_question, AT_IARF,
+                  "Add or remove space around the '?' in 'b ? t : f'");
+
+
    unc_begin_group(UG_indent, "Indenting");
    unc_add_option("indent_columns", UO_indent_columns, AT_NUM,
                   "The number of columns to indent per level.\n"
@@ -331,6 +347,9 @@ void register_options(void)
                   "Whether the 'class' body is indented");
    unc_add_option("indent_class_colon", UO_indent_class_colon, AT_BOOL,
                   "Whether to indent the stuff after a leading class colon");
+   unc_add_option("indent_else_if", UO_indent_else_if, AT_BOOL,
+                  "False=treat 'else\\nif' as 'else if' for indenting purposes\n"
+                  "True=indent the 'if' one level\n");
 
    unc_add_option("indent_func_call_param", UO_indent_func_call_param, AT_BOOL,
                   "True:  indent continued function call parameters one indent level\n"
@@ -343,6 +362,8 @@ void register_options(void)
                   "Same as indent_func_call_param, but for class declarations");
    unc_add_option("indent_func_ctor_var_param", UO_indent_func_ctor_var_param, AT_BOOL,
                   "Same as indent_func_call_param, but for class variable constructors");
+   unc_add_option("indent_template_param", UO_indent_template_param, AT_BOOL,
+                  "Same as indent_func_call_param, but for templates");
    unc_add_option("indent_func_param_double", UO_indent_func_param_double, AT_BOOL,
                   "Double the indent for indent_func_xxx_param options");
 
@@ -444,6 +465,8 @@ void register_options(void)
                   "If set to ignore, nl_if_brace is used instead");
    unc_add_option("nl_else_brace", UO_nl_else_brace, AT_IARF,
                   "Add or remove newline between 'else' and '{'");
+   unc_add_option("nl_else_if", UO_nl_else_if, AT_IARF,
+                  "Add or remove newline between 'else' and 'if'");
    unc_add_option("nl_brace_finally", UO_nl_brace_finally, AT_IARF,
                   "Add or remove newline between '}' and 'finally'");
    unc_add_option("nl_finally_brace", UO_nl_finally_brace, AT_IARF,
@@ -479,7 +502,7 @@ void register_options(void)
    unc_add_option("nl_namespace_brace", UO_nl_namespace_brace, AT_IARF,
                   "Newline between namespace and {");
    unc_add_option("nl_template_class", UO_nl_template_class, AT_IARF,
-                  "Add or remove newline between 'template<>' and 'class'");
+                  "Add or remove newline between 'template<>' and whatever follows.");
    unc_add_option("nl_class_brace", UO_nl_class_brace, AT_IARF,
                   "Add or remove newline between 'class' and '{'");
    unc_add_option("nl_class_init_args", UO_nl_class_init_args, AT_IARF,
@@ -500,6 +523,8 @@ void register_options(void)
                   "Add or remove newline between function signature and '{'");
    unc_add_option("nl_after_return", UO_nl_after_return, AT_BOOL,
                   "Whether to put a newline after 'return' statement");
+   unc_add_option("nl_return_expr", UO_nl_return_expr, AT_IARF,
+                  "Add or remove a newline between the return keyword and return expression.");
    unc_add_option("nl_after_semicolon", UO_nl_after_semicolon, AT_BOOL,
                   "Whether to put a newline after semicolons, except in 'for' statements");
    unc_add_option("nl_after_brace_open", UO_nl_after_brace_open, AT_BOOL,
@@ -617,6 +642,9 @@ void register_options(void)
                   "Whether to left-align numbers");
    unc_add_option("align_func_params", UO_align_func_params, AT_BOOL,
                   "Align variable definitions in prototypes and functions");
+   unc_add_option("align_same_func_call_params", UO_align_same_func_call_params, AT_BOOL,
+                  "Align parameters in single-line functions that have the same name.\n"
+                  "The function names must already be aligned with each other.");
    unc_add_option("align_var_def_span", UO_align_var_def_span, AT_NUM,
                   "The span for aligning variable definitions (0=don't align)", "", 0, 5000);
    unc_add_option("align_var_def_star_style", UO_align_var_def_star_style, AT_NUM,
@@ -672,10 +700,15 @@ void register_options(void)
                   "0: Align on typdef type, ignore '&'\n"
                   "1: The '&' is part of type name: typedef int  &pint;\n"
                   "2: The '&' is part of the type, but dangling: typedef int &pint;", "", 0, 2);
+
    unc_add_option("align_right_cmt_span", UO_align_right_cmt_span, AT_NUM,
                   "The span for aligning comments that end lines (0=don't align)", "", 0, 5000);
    unc_add_option("align_right_cmt_mix", UO_align_right_cmt_mix, AT_BOOL,
                   "If aligning comments, mix with comments after '}' and #endif with less than 3 spaces before the comment");
+   unc_add_option("align_right_cmt_gap", UO_align_right_cmt_gap, AT_NUM,
+                  "If a trailing comment is more than this number of columns away from the text it follows,\n"
+                  "it will qualify for being aligned.");
+
    unc_add_option("align_func_proto_span", UO_align_func_proto_span, AT_NUM,
                   "The span for aligning function prototypes (0=don't align)", "", 0, 5000);
    unc_add_option("align_func_proto_gap", UO_align_func_proto_gap, AT_NUM,
@@ -704,7 +737,8 @@ void register_options(void)
    unc_add_option("cmt_width", UO_cmt_width, AT_NUM,
                   "Try to wrap comments at cmt_width columns", "", 16, 256);
    unc_add_option("cmt_indent_multi", UO_cmt_indent_multi, AT_BOOL,
-                  "Whether to mess with the indent of multi-line comments");
+                  "If false, disable all multi-line comment changes, including cmt_width and leading chars.\n"
+                  "Default is true.");
    unc_add_option("cmt_c_group", UO_cmt_c_group, AT_BOOL,
                   "Whether to group c-comments that look like they are in a block");
    unc_add_option("cmt_c_nl_start", UO_cmt_c_nl_start, AT_BOOL,
@@ -726,6 +760,10 @@ void register_options(void)
    unc_add_option("cmt_sp_after_star_cont", UO_cmt_sp_after_star_cont, AT_NUM,
                   "The number of spaces to insert after the star on subsequent comment lines");
 
+   unc_add_option("cmt_multi_check_last", UO_cmt_multi_check_last, AT_BOOL,
+                  "For multi-line comments with a '*' lead, remove leading spaces if the first and last lines of\n"
+                  "the comment are the same length. Default=True");
+
    unc_add_option("cmt_insert_file_header", UO_cmt_insert_file_header, AT_STRING,
                   "The filename that contains text to insert at the head of a file if the file doesn't start with a C/C++ comment.\n"
                   "Will substitue $(filename) with the current file's name.");
@@ -745,7 +783,7 @@ void register_options(void)
    unc_add_option("mod_full_brace_function", UO_mod_full_brace_function, AT_IARF,
                   "Add or remove braces on single-line function defintions. (Pawn)");
    unc_add_option("mod_full_brace_if", UO_mod_full_brace_if, AT_IARF,
-                  "Add or remove braces on single-line 'if' statement");
+                  "Add or remove braces on single-line 'if' statement. Will not remove the braces if they contain an 'else'.");
    unc_add_option("mod_full_brace_nl", UO_mod_full_brace_nl, AT_NUM,
                   "Don't remove braces around statements that span N newlines", "", 0, 5000);
    unc_add_option("mod_full_brace_while", UO_mod_full_brace_while, AT_IARF,
@@ -779,6 +817,8 @@ void register_options(void)
                   "Control indent of preprocessors inside #if blocks at brace level 0");
    unc_add_option("pp_indent_at_level", UO_pp_indent_at_level, AT_BOOL,
                   "Whether to indent #if/#else/#endif at the brace level (true) or from column 1 (false)");
+   unc_add_option("pp_indent_count", UO_pp_indent_count, AT_NUM,
+                  "If pp_indent_at_level=false, specifies the number of columns to indent per level. Default=1.");
    unc_add_option("pp_space", UO_pp_space, AT_IARF,
                   "Add or remove space after # based on pp_level of #if blocks");
    unc_add_option("pp_space_count", UO_pp_space_count, AT_NUM,
@@ -1303,24 +1343,27 @@ void print_options(FILE *pfile, bool verbose)
  */
 void set_option_defaults(void)
 {
-   cpd.settings[UO_newlines].le          = LE_AUTO;
-   cpd.settings[UO_input_tab_size].n     = 8;
-   cpd.settings[UO_output_tab_size].n    = 8;
-   cpd.settings[UO_indent_columns].n     = 8;
-   cpd.settings[UO_indent_with_tabs].n   = 1;
-   cpd.settings[UO_indent_label].n       = 1;
-   cpd.settings[UO_indent_access_spec].n = 1;
-   cpd.settings[UO_sp_before_comma].a    = AV_REMOVE;
-   cpd.settings[UO_string_escape_char].n = '\\';
-   cpd.settings[UO_sp_not].a             = AV_REMOVE;
-   cpd.settings[UO_sp_inv].a             = AV_REMOVE;
-   cpd.settings[UO_sp_addr].a            = AV_REMOVE;
-   cpd.settings[UO_sp_deref].a           = AV_REMOVE;
-   cpd.settings[UO_sp_member].a          = AV_REMOVE;
-   cpd.settings[UO_sp_sign].a            = AV_REMOVE;
-   cpd.settings[UO_sp_incdec].a          = AV_REMOVE;
-   cpd.settings[UO_sp_after_type].a      = AV_FORCE;
-   cpd.settings[UO_cmt_indent_multi].b   = true;
+   cpd.settings[UO_newlines].le            = LE_AUTO;
+   cpd.settings[UO_input_tab_size].n       = 8;
+   cpd.settings[UO_output_tab_size].n      = 8;
+   cpd.settings[UO_indent_columns].n       = 8;
+   cpd.settings[UO_indent_with_tabs].n     = 1;
+   cpd.settings[UO_indent_label].n         = 1;
+   cpd.settings[UO_indent_access_spec].n   = 1;
+   cpd.settings[UO_sp_before_comma].a      = AV_REMOVE;
+   cpd.settings[UO_string_escape_char].n   = '\\';
+   cpd.settings[UO_sp_not].a               = AV_REMOVE;
+   cpd.settings[UO_sp_inv].a               = AV_REMOVE;
+   cpd.settings[UO_sp_addr].a              = AV_REMOVE;
+   cpd.settings[UO_sp_deref].a             = AV_REMOVE;
+   cpd.settings[UO_sp_member].a            = AV_REMOVE;
+   cpd.settings[UO_sp_sign].a              = AV_REMOVE;
+   cpd.settings[UO_sp_incdec].a            = AV_REMOVE;
+   cpd.settings[UO_sp_after_type].a        = AV_FORCE;
+   cpd.settings[UO_sp_before_nl_cont].a    = AV_ADD;
+   cpd.settings[UO_cmt_indent_multi].b     = true;
+   cpd.settings[UO_cmt_multi_check_last].b = true;
+   cpd.settings[UO_pp_indent_count].n      = 1;
 }
 
 std::string argtype_to_string(argtype_e argtype)
