@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: options.h 1192 2008-06-06 19:12:54Z bengardner $
+ * $Id: options.h 1300 2008-07-18 22:17:50Z bengardner $
  */
 #ifndef OPTIONS_H_INCLUDED
 #define OPTIONS_H_INCLUDED
@@ -104,6 +104,7 @@ enum uncrustify_options
    UO_indent_bool_paren,         // indent of bool if inside a paren
    UO_pp_indent,                 // indent preproc 1 space per level (add/ignore/remove)
    UO_pp_indent_at_level,        // indent #if, #else, #endif at brace level
+   UO_pp_indent_count,
    UO_pp_define_at_level,        // indent #define at brace level
    UO_pp_space,                  // spaces between # and word (add/ignore/remove)
    UO_pp_space_count,            // the number of spaces for add/force
@@ -134,6 +135,7 @@ enum uncrustify_options
    UO_indent_func_def_param,     // same, but for function defs
    UO_indent_func_class_param,   // same, but for classes
    UO_indent_func_ctor_var_param,
+   UO_indent_template_param,
    UO_indent_func_param_double,  // double the tab indent for
 
    UO_indent_namespace,          // indent stuff inside namespace braces
@@ -147,6 +149,8 @@ enum uncrustify_options
    UO_indent_relative_single_line_comments, // indent single line ('//') comments after code
    UO_indent_preserve_sql,       // preserve indent of EXEC SQL statement body
 
+   UO_indent_else_if,
+
    /*
     * Misc inter-element spacing
     */
@@ -155,7 +159,7 @@ enum uncrustify_options
    UO_sp_fparen_brace,          // space between ')' and '{' of function
    UO_sp_sparen_brace,          // space between ')' and '{' of if, while, etc
 
-   UO_sp_after_cast,            // space after cast - "(int) a" vs "(int)a"
+   UO_sp_after_cast,            // space after C & D cast - "(int) a" vs "(int)a"
    UO_sp_inside_paren_cast,     // spaces inside the parens of a cast
 
    UO_sp_before_byref,          // space before '&' of 'fcn(int& idx)'
@@ -174,6 +178,7 @@ enum uncrustify_options
    UO_sp_before_sparen,         // space before '(' of 'if/for/while/switch'
    UO_sp_after_sparen,      /* space after  ')' of 'if/for/while/switch'
                              * the do-while does not get set here */
+   UO_sp_template_angle,
    UO_sp_before_angle,          // space before '<>', as in '<class T>'
    UO_sp_after_angle,           // space after  '<>', as in '<class T>'
    UO_sp_angle_paren,           // space between '<>' and '(' in "a = new List<byte>();"
@@ -206,6 +211,7 @@ enum uncrustify_options
    UO_sp_func_class_paren,      // space between ctor/dtor and '('
 
    UO_sp_attribute_paren,       // space between '__attribute__' and '('
+   UO_sp_defined_paren,
 
    UO_sp_type_func,             // space between return type and 'func'
    // a minimum of 1 is forced except for '*'
@@ -232,6 +238,7 @@ enum uncrustify_options
    UO_sp_after_tag,             // pawn: space after a tag colon
 
    UO_sp_after_operator,        // space after operator when followed by a punctuator
+   UO_sp_after_operator_sym,    // space after operator when followed by a punctuator
    UO_sp_else_brace,
    UO_sp_brace_else,
 
@@ -252,9 +259,12 @@ enum uncrustify_options
    UO_sp_member,
    UO_sp_sign,
    UO_sp_incdec,
+   UO_sp_before_nl_cont,
    UO_sp_after_oc_scope,
    UO_sp_before_oc_colon,
    UO_sp_after_oc_type,
+   UO_sp_cond_colon,
+   UO_sp_cond_question,
 
    /*
     * Line splitting options (for long lines)
@@ -283,7 +293,9 @@ enum uncrustify_options
    UO_align_assign_thresh,        // threshold for aligning on '='. 0=no limit
    UO_align_right_cmt_span,       // align comment that end lines. 0=don't align
    UO_align_right_cmt_mix,        // mix comments after '}' and preproc with others
+   UO_align_right_cmt_gap,
    UO_align_func_params,          // align prototype variable defs on variable
+   UO_align_same_func_call_params,
    UO_align_var_def_span,         // align variable defs on variable (span for regular stuff)
    UO_align_var_def_thresh,       // align variable defs threshold
    UO_align_var_def_gap,          // align variable defs gap
@@ -336,6 +348,7 @@ enum uncrustify_options
    UO_nl_before_case,             // newline before 'case' statement
    UO_nl_before_throw,
    UO_nl_after_return,            /* newline after return statement */
+   UO_nl_return_expr,
    UO_nl_after_case,              /* disallow nested "case 1: a=3;" */
    UO_nl_after_semicolon,         // disallow multiple statements on a line "a=1;b=4;"
    UO_nl_after_brace_open,        // force a newline after a brace open
@@ -355,6 +368,7 @@ enum uncrustify_options
    UO_nl_do_brace,                /* nl between do and { */
    UO_nl_if_brace,                /* nl between if and { */
    UO_nl_for_brace,               /* nl between for and { */
+   UO_nl_else_if,
    UO_nl_else_brace,              /* nl between else and { */
    UO_nl_finally_brace,           /* nl between finally and { */
    UO_nl_brace_finally,           /* nl between } and finally */
@@ -473,6 +487,7 @@ enum uncrustify_options
    UO_cmt_c_nl_end,             // put a nl before the */ in a combined group
    UO_cmt_cpp_nl_start,         // put a blank /* at the start of a converted group
    UO_cmt_cpp_nl_end,           // put a nl before the */ in a converted group
+   UO_cmt_multi_check_last,
 
    UO_cmt_insert_file_header,
    UO_cmt_insert_func_header,
