@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: output.cpp 1191 2008-06-06 17:55:28Z bengardner $
+ * $Id: output.cpp 1290 2008-07-18 00:03:31Z bengardner $
  */
 
 #include "uncrustify_types.h"
@@ -15,6 +15,8 @@
 #include <cstring>
 #include <cstdlib>
 
+static void output_comment_multi(chunk_t *pc);
+static void output_comment_multi_simple(chunk_t *pc);
 
 struct cmt_reflow
 {
@@ -33,7 +35,7 @@ static void add_comment_text(const char *text, int len,
                              cmt_reflow& cmt, bool esc_close);
 
 
-void add_char(char ch)
+static void add_char(char ch)
 {
    static char last_char = 0;
 
@@ -72,7 +74,7 @@ void add_char(char ch)
    last_char = ch;
 }
 
-void add_text(const char *text)
+static void add_text(const char *text)
 {
    char ch;
 
@@ -83,7 +85,7 @@ void add_text(const char *text)
    }
 }
 
-void add_text_len(const char *text, int len)
+static void add_text_len(const char *text, int len)
 {
    while (len-- > 0)
    {
@@ -98,7 +100,7 @@ void add_text_len(const char *text, int len)
  *
  * @param column  The column to advance to
  */
-void output_to_column(int column, bool allow_tabs)
+static void output_to_column(int column, bool allow_tabs)
 {
    int nc;
 
@@ -118,8 +120,15 @@ void output_to_column(int column, bool allow_tabs)
    }
 }
 
-void output_indent(int column, int brace_col)
+/**
+ * Output to the column using indent_with_tabs as the rule.
+ *
+ * @param column    the column that we should end up in
+ * @param brace_col the level that indent_with_tabs=1 should tab to
+ */
+static void output_indent(int column, int brace_col)
 {
+   cpd.did_newline = 0;
    if ((cpd.column == 1) && (cpd.settings[UO_indent_with_tabs].n != 0))
    {
       if (cpd.settings[UO_indent_with_tabs].n == 2)
@@ -246,7 +255,14 @@ void output_text(FILE *pfile)
       }
       else if (pc->type == CT_COMMENT_MULTI)
       {
-         output_comment_multi(pc);
+         if (cpd.settings[UO_cmt_indent_multi].b)
+         {
+            output_comment_multi(pc);
+         }
+         else
+         {
+            output_comment_multi_simple(pc);
+         }
       }
       else if (pc->type == CT_COMMENT_CPP)
       {
@@ -271,7 +287,8 @@ void output_text(FILE *pfile)
                /* FIXME: it would be better to properly set column_indent in
                 * indent_text(), but this hack for '}' and ':' seems to work. */
                if ((pc->type == CT_BRACE_CLOSE) ||
-                   chunk_is_str(pc, ":", 1))
+                   chunk_is_str(pc, ":", 1) ||
+                   (pc->type == CT_PREPROC))
                {
                   lvlcol = pc->column;
                }
@@ -367,18 +384,21 @@ static int calculate_comment_body_indent(const char *str, int len, int start_col
       return(0);
    }
 
-   /* find the last line length */
-   for (idx = len - 1; idx > 0; idx--)
+   if (cpd.settings[UO_cmt_multi_check_last].b)
    {
-      if ((str[idx] == '\n') || (str[idx] == '\r'))
+      /* find the last line length */
+      for (idx = len - 1; idx > 0; idx--)
       {
-         idx++;
-         while ((idx < len) && ((str[idx] == ' ') || (str[idx] == '\t')))
+         if ((str[idx] == '\n') || (str[idx] == '\r'))
          {
             idx++;
+            while ((idx < len) && ((str[idx] == ' ') || (str[idx] == '\t')))
+            {
+               idx++;
+            }
+            last_len = len - idx;
+            break;
          }
-         last_len = len - idx;
-         break;
       }
    }
 
@@ -447,28 +467,6 @@ static int calculate_comment_body_indent(const char *str, int len, int start_col
    return((width == 2) ? 0 : 1);
 }
 
-static_inline void add_spaces_before_star()
-{
-   int count = cpd.settings[UO_cmt_sp_before_star_cont].n;
-
-   while (count-- > 0)
-   {
-      add_char(' ');
-   }
-}
-
-static_inline void add_spaces_after_star()
-{
-   if (cpd.settings[UO_cmt_star_cont].b)
-   {
-      int count = cpd.settings[UO_cmt_sp_after_star_cont].n;
-      while (count-- > 0)
-      {
-         add_char(' ');
-      }
-   }
-}
-
 static chunk_t *get_next_function(chunk_t *pc)
 {
    while ((pc = chunk_get_next(pc)) != NULL)
@@ -889,7 +887,7 @@ static chunk_t *output_comment_cpp(chunk_t *first)
  * The only trick here is that we have to trim out whitespace characters
  * to get the comment to line up.
  */
-void output_comment_multi(chunk_t *pc)
+static void output_comment_multi(chunk_t *pc)
 {
    int        cmt_col = pc->column;
    const char *cmt_str;
@@ -1004,9 +1002,7 @@ void output_comment_multi(chunk_t *pc)
          if (line_count == 1)
          {
             /* this is the first line - add unchanged */
-
-            /*TODO: need to support indent_with_tabs mode 1 */
-            output_to_column(cmt_col, cpd.settings[UO_indent_with_tabs].n != 0);
+            output_indent(cmt_col, cmt_col);
             add_comment_text(line, line_len, cmt, false);
             if (nl_end)
             {
@@ -1016,22 +1012,24 @@ void output_comment_multi(chunk_t *pc)
          else
          {
             /* This is not the first line, so we need to indent to the
-             * correct column.
+             * correct column. Each line is indented a minimum of three spaces.
+             * Those three spaces are filled with whatever lead char should be
+             * there.
              */
             ccol -= col_diff;
-            if (ccol < cmt_col)
+            if (ccol < (cmt_col + 3))
             {
-               ccol = cmt_col;
+               ccol = cmt_col + 3;
             }
 
-            if (line[0] == 0)
+            if (line_len == 0)
             {
                /* Emtpy line - just a '\n' */
                if (cpd.settings[UO_cmt_star_cont].b)
                {
-                  output_to_column(cmt_col, cpd.settings[UO_indent_with_tabs].n != 0);
-                  add_spaces_before_star();
-                  add_text((xtra == 1) ? " *" : "*");
+                  output_indent(cmt_col + xtra + cpd.settings[UO_cmt_sp_before_star_cont].n,
+                                cmt_col);
+                  add_char('*');
                }
                add_char('\n');
             }
@@ -1044,30 +1042,24 @@ void output_comment_multi(chunk_t *pc)
                    (line[0] != '*') && (line[0] != '|') && (line[0] != '#') &&
                    ((line[0] != '\\') || unc_isalpha(line[1])) && (line[0] != '+'))
                {
-                  bool do_sp_add = false;
-                  output_to_column(cmt_col, cpd.settings[UO_indent_with_tabs].n != 0);
-                  add_spaces_before_star();
+                  int start_col = cmt_col + xtra + cpd.settings[UO_cmt_sp_before_star_cont].n;
 
                   if (cpd.settings[UO_cmt_star_cont].b)
                   {
-                     cmt.cont_text = (xtra == 1) ? " * " : "*  ";
-                     do_sp_add = true;
+                     output_indent(start_col, cmt_col);
+                     add_text("* ");
+                     output_to_column(ccol + cpd.settings[UO_cmt_sp_after_star_cont].n,
+                                      false);
                   }
                   else
                   {
-                     cmt.cont_text = "   ";
+                     output_indent(ccol, cmt_col);
                   }
-                  add_text(cmt.cont_text);
-                  if (do_sp_add)
-                  {
-                     add_spaces_after_star();
-                  }
-                  output_to_column(ccol, cpd.settings[UO_indent_with_tabs].n != 0);
                }
                else
                {
-                  output_to_column(cmt_col + xtra, cpd.settings[UO_indent_with_tabs].n != 0);
-                  add_spaces_before_star();
+                  output_indent(cmt_col + xtra + cpd.settings[UO_cmt_sp_before_star_cont].n,
+                                cmt_col);
 
                   int idx  = 0;
                   int sidx = 0;
@@ -1095,3 +1087,134 @@ void output_comment_multi(chunk_t *pc)
       }
    }
 }
+
+/**
+ * Output a multiline comment without any reformatting other than shifting
+ * it left or right to get the column right.
+ * Oh, and trim trailing whitespace.
+ */
+static void output_comment_multi_simple(chunk_t *pc)
+{
+   int        cmt_col = pc->column;
+   const char *cmt_str;
+   int        remaining;
+   char       ch;
+   char       line[1024];
+   int        line_len;
+   int        line_count = 0;
+   int        ccol;
+   int        col_diff = 0;
+   bool       nl_end = false;
+
+   if (chunk_is_newline(chunk_get_prev(pc)))
+   {
+      /* The comment should be indented correctly */
+      cmt_col  = pc->column;
+      col_diff = pc->orig_col - pc->column;
+   }
+   else
+   {
+      /* The comment starts after something else */
+      cmt_col  = pc->orig_col;
+      col_diff = 0;
+   }
+
+   //LOG_FMT(LSYS, "Indenting1 line %d to col %d (orig=%d) col_diff=%d xtra=%d\n",
+   //        pc->orig_line, cmt_col, pc->orig_col, col_diff, xtra);
+
+   ccol      = pc->column;
+   remaining = pc->len;
+   cmt_str   = pc->str;
+   line_len  = 0;
+   while (remaining > 0)
+   {
+      ch = *cmt_str;
+      cmt_str++;
+      remaining--;
+
+      /* handle the CRLF and CR endings. convert both to LF */
+      if (ch == '\r')
+      {
+         ch = '\n';
+         if (*cmt_str == '\n')
+         {
+            cmt_str++;
+            remaining--;
+         }
+      }
+
+      /* Find the start column */
+      if (line_len == 0)
+      {
+         nl_end = false;
+         if (ch == ' ')
+         {
+            ccol++;
+            continue;
+         }
+         else if (ch == '\t')
+         {
+            ccol = calc_next_tab_column(ccol, cpd.settings[UO_input_tab_size].n);
+            continue;
+         }
+         else
+         {
+            //LOG_FMT(LSYS, "%d] Text starts in col %d, col_diff=%d, real=%d\n",
+            //        line_count, ccol, col_diff, ccol - col_diff);
+         }
+      }
+
+      line[line_len++] = ch;
+
+      /* If we just hit an end of line OR we just hit end-of-comment... */
+      if ((ch == '\n') || (remaining == 0))
+      {
+         line_count++;
+
+         /* strip trailing tabs and spaces before the newline */
+         if (ch == '\n')
+         {
+            line_len--;
+            while ((line_len > 0) &&
+                   ((line[line_len - 1] == ' ') ||
+                    (line[line_len - 1] == '\t')))
+            {
+               line_len--;
+            }
+            /* REVISIT: the check for '*' seem unnecessary */
+            if ((line[line_len - 1] == '\\') && (line[line_len - 2] != '*'))
+            {
+               /* Kill off the backslash-newline */
+               line_len--;
+               while ((line_len > 0) &&
+                      ((line[line_len - 1] == ' ') ||
+                       (line[line_len - 1] == '\t')))
+               {
+                  line_len--;
+               }
+               line[line_len++] = ' ';
+               line[line_len++] = '\\';
+            }
+            nl_end = true;
+         }
+         line[line_len] = 0;
+
+         if (line_count > 1)
+         {
+            ccol -= col_diff;
+         }
+
+         if (line_len > 0)
+         {
+            output_indent(ccol, cmt_col);
+            add_text_len(line, line_len);
+         }
+         if (nl_end)
+         {
+            add_char('\n');
+         }
+         line_len = 0;
+         ccol     = 1;
+      }
+   }
+}
