@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: parens.cpp 952 2007-10-02 20:53:21Z bengardner $
+ * $Id: parens.cpp 1088 2008-01-30 01:24:42Z bengardner $
  */
 #include "uncrustify_types.h"
 #include "chunk_list.h"
@@ -14,7 +14,7 @@
 #include <cstdlib>
 #include <cstring>
 #include <cerrno>
-#include <cctype>
+#include "unc_ctype.h"
 
 
 static void add_parens_between(chunk_t *first, chunk_t *last);
