@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: parse_frame.cpp 987 2007-11-02 18:53:04Z bengardner $
+ * $Id: parse_frame.cpp 1074 2008-01-15 01:59:50Z bengardner $
  */
 
 #include "uncrustify_types.h"
@@ -22,9 +22,11 @@ void pf_log(int logsev, struct parse_frame *pf)
 {
    int idx;
 
-   //LOG_FMT(logsev, "BrLevel=%d Level=%d PseTos=%d\n", pf->brace_level,
-   //        pf->level, pf->pse_tos);
+   LOG_FMT(logsev, "[%s] BrLevel=%d Level=%d PseTos=%d\n",
+           get_token_name(pf->in_ifdef),
+           pf->brace_level, pf->level, pf->pse_tos);
 
+   LOG_FMT(logsev, " *");
    for (idx = 1; idx <= pf->pse_tos; idx++)
    {
       LOG_FMT(logsev, " [%s-%d]",
@@ -34,6 +36,20 @@ void pf_log(int logsev, struct parse_frame *pf)
    LOG_FMT(logsev, "\n");
 }
 
+static void pf_log_frms(int logsev, const char *txt, struct parse_frame *pf)
+{
+   int idx;
+
+   LOG_FMT(logsev, "%s Parse Frames(%d):", txt, cpd.frame_count);
+   for (idx = 0; idx < cpd.frame_count; idx++)
+   {
+      LOG_FMT(logsev, " [%s-%d]",
+              get_token_name(cpd.frames[idx].in_ifdef),
+              cpd.frames[idx].ref_no);
+   }
+   LOG_FMT(logsev, "-[%s-%d]\n", get_token_name(pf->in_ifdef), pf->ref_no);
+}
+
 /**
  * Logs the entire parse frame stack
  */
@@ -45,10 +61,11 @@ void pf_log_all(int logsev)
 
    for (idx = 0; idx < cpd.frame_count; idx++)
    {
-      LOG_FMT(logsev, " <%d> ", idx);
+      LOG_FMT(logsev, "##  <%d> ", idx);
 
       pf_log(logsev, &cpd.frames[idx]);
    }
+   LOG_FMT(logsev, "##=-\n");
 }
 
 /**
@@ -65,10 +82,13 @@ void pf_copy(struct parse_frame *dst, const struct parse_frame *src)
  */
 void pf_push(struct parse_frame *pf)
 {
+   static int ref_no = 1;
+
    if (cpd.frame_count < (int)ARRAY_SIZE(cpd.frames))
    {
       pf_copy(&cpd.frames[cpd.frame_count], pf);
       cpd.frame_count++;
+      pf->ref_no = ref_no++;
    }
    LOG_FMT(LPF, "%s: count = %d\n", __func__, cpd.frame_count);
 }
@@ -111,6 +131,21 @@ void pf_copy_tos(struct parse_frame *pf)
    LOG_FMT(LPF, "%s: count = %d\n", __func__, cpd.frame_count);
 }
 
+/**
+ * Copy the 2nd top item off the stack into pf.
+ * This is called on #else and #elif.
+ * The stack contains [...] [base] [if] at this point.
+ * We want to copy [base].
+ */
+static void pf_copy_2nd_tos(struct parse_frame *pf)
+{
+   if (cpd.frame_count > 1)
+   {
+      pf_copy(pf, &cpd.frames[cpd.frame_count - 2]);
+   }
+   LOG_FMT(LPF, "%s: count = %d\n", __func__, cpd.frame_count);
+}
+
 /**
  * Deletes the top frame from the stack.
  */
@@ -154,6 +189,7 @@ int pf_check(struct parse_frame *frm, chunk_t *pc)
 
    LOG_FMT(LPFCHK, "%s: %5d] %s\n",
            __func__, pc->orig_line, get_token_name(pc->parent_type));
+   pf_log_frms(LPFCHK, "TOP", frm);
 
    if ((pc->flags & PCF_IN_PREPROC) != 0)
    {
@@ -162,6 +198,7 @@ int pf_check(struct parse_frame *frm, chunk_t *pc)
 
       if (pc->parent_type == CT_PP_IF)
       {
+         /* An #if pushes a copy of the current frame on the stack */
          cpd.pp_level++;
          pf_push(frm);
          frm->in_ifdef = CT_PP_IF;
@@ -170,38 +207,52 @@ int pf_check(struct parse_frame *frm, chunk_t *pc)
       else if (pc->parent_type == CT_PP_ELSE)
       {
          pp_level--;
+         /**
+          * For #else of #elif, we want to keep the #if part and throw out the
+          * else parts.
+          * We check to see what the top type is to see if we just push or
+          * pop and then push.
+          * We need to use the copy right before the if.
+          */
          if (frm->in_ifdef == CT_PP_IF)
          {
-            /* need to switch */
-            pf_push_under(frm);
-            pf_copy_tos(frm);
+            /* we have [...] [base]-[if], so push an [else] */
+            pf_push(frm);
             frm->in_ifdef = CT_PP_ELSE;
-            txt           = "else-push_under";
-         }
-         else if (frm->in_ifdef == CT_PP_ELSE)
-         {
-            pf_copy_tos(frm);
-            frm->in_ifdef = CT_PP_ELSE;
-            txt           = "else-copy";
-         }
-         else
-         {
-            txt = "???";
          }
+         /* we have [...] [base] [if]-[else], copy [base] over [else] */
+         pf_copy_2nd_tos(frm);
+         frm->in_ifdef = CT_PP_ELSE;
+         txt           = "else-push";
       }
       else if (pc->parent_type == CT_PP_ENDIF)
       {
+         /**
+          * we may have [...] [base] [if]-[else] or [...] [base]-[if].
+          * Throw out the [else].
+          */
          cpd.pp_level--;
          pp_level--;
 
          if (frm->in_ifdef == CT_PP_ELSE)
          {
-            pf_trash_tos();
-            pf_pop(frm);
+            /**
+             * We have: [...] [base] [if]-[else]
+             * We want: [...]-[if]
+             */
+            pf_copy_tos(frm);     /* [...] [base] [if]-[if] */
+            frm->in_ifdef = cpd.frames[cpd.frame_count - 2].in_ifdef;
+            pf_trash_tos();       /* [...] [base]-[if] */
+            pf_trash_tos();       /* [...]-[if] */
+
             txt = "endif-trash/pop";
          }
          else if (frm->in_ifdef == CT_PP_IF)
          {
+            /**
+             * We have: [...] [base] [if]
+             * We want: [...] [base]
+             */
             pf_pop(frm);
             txt = "endif-pop";
          }
@@ -222,5 +273,7 @@ int pf_check(struct parse_frame *frm, chunk_t *pc)
       pf_log(LPF, frm);
    }
 
+   pf_log_frms(LPFCHK, "END", frm);
+
    return(pp_level);
 }
