@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: token_enum.h 1169 2008-05-02 01:19:53Z bengardner $
+ * $Id: token_enum.h 1286 2008-07-17 23:16:05Z bengardner $
  */
 #ifndef TOKEN_ENUM_H_INCLUDED
 #define TOKEN_ENUM_H_INCLUDED
@@ -38,6 +38,8 @@ typedef enum
    CT_COMMENT_END,   /* comment parent_type: newline after */
    CT_COMMENT_WHOLE, /* comment parent_type: newline before and after */
 
+   CT_IGNORED,       /* a chunk of ignored text */
+
    CT_WORD,          /* variable, type, function name, etc */
    CT_NUMBER,
    CT_NUMBER_FP,
@@ -62,7 +64,9 @@ typedef enum
    CT_UNION,
    CT_GOTO,
    CT_CONTINUE,
-   CT_CAST,                /* cast(exp) -- as used in D */
+   CT_C_CAST,              /* C-style cast:   "(int)5.6" */
+   CT_CPP_CAST,            /* C++-style cast: "int(5.6)" */
+   CT_D_CAST,              /* D-style cast:   "cast(type)" */
    CT_TYPE_CAST,           /* static_cast<type>(exp) */
    CT_TYPENAME,            /* typename type */
    CT_TEMPLATE,            /* template<...> */
@@ -109,7 +113,7 @@ typedef enum
    CT_CASE_COLON,
    CT_CLASS_COLON,         /* colon after a class def or constructor */
    CT_D_ARRAY_COLON,       /* D named array initializer colon */
-   CT_Q_COLON,
+   CT_COND_COLON,          /* conditional colon in  'b ? t : f' */
    CT_QUESTION,
    CT_COMMA,
 
