@@ -5,7 +5,7 @@
  * @author  Ben Gardner
  * @license GPL v2+
  *
- * $Id: uncrustify.cpp 979 2007-10-25 02:11:52Z bengardner $
+ * $Id: uncrustify.cpp 1164 2008-04-26 04:49:27Z bengardner $
  */
 #define DEFINE_PCF_NAMES
 #define DEFINE_CHAR_TABLE
@@ -26,12 +26,12 @@
 #include <cstdlib>
 #include <cstring>
 #include <cerrno>
-#include <cctype>
+#include "unc_ctype.h"
 #ifdef HAVE_SYS_STAT_H
-#include <sys/stat.h>
+ #include <sys/stat.h>
 #endif
 #ifdef HAVE_STRINGS_H
-#include <strings.h> /* strcasecmp() */
+ #include <strings.h> /* strcasecmp() */
 #endif
 
 /* Global data */
@@ -44,12 +44,20 @@ static const char *language_to_string(int lang);
 static char *read_stdin(int& out_len);
 static void uncrustify_file(const char *data, int data_len, FILE *pfout,
                             const char *parsed_file);
-static void do_source_file(const char *filename, FILE *pfout, const char *parsed_file,
-                           const char *prefix, const char *suffix, bool no_backup);
+static void do_source_file(const char *filename_in,
+                           const char *filename_out,
+                           const char *parsed_file,
+                           bool no_backup, bool keep_mtime);
 static void process_source_list(const char *source_list, const char *prefix,
-                                const char *suffix, bool no_backup);
+                                const char *suffix, bool no_backup, bool keep_mtime);
 static int load_header_files();
 
+static const char *make_output_filename(char *buf, int buf_size,
+                                        const char *filename,
+                                        const char *prefix,
+                                        const char *suffix);
+
+
 /**
  * Replace the brain-dead and non-portable basename().
  * Returns a pointer to the character after the last '/'.
@@ -103,15 +111,29 @@ static void usage_exit(const char *msg, const char *argv0, int code)
    {
       fprintf(stderr, "%s\n", msg);
    }
-   if (code != EXIT_SUCCESS)
+   if ((code != EXIT_SUCCESS) || (argv0 == NULL))
    {
       fprintf(stderr, "Try running with -h for usage information\n");
       exit(code);
    }
-   fprintf(stderr,
+   fprintf(stdout,
            "Usage:\n"
            "%s [options] [files ...]\n"
            "\n"
+           "If no input files are specified, the input is read from stdin\n"
+           "If reading from stdin, you should specify the language using -l\n"
+           "\n"
+           "If -F is used or files are specified on the command line, the output filename is\n"
+           "'prefix/filename' + suffix\n"
+           "\n"
+           "When reading from stdin or doing a single file via the '-f' option,\n"
+           "the output is dumped to stdout, unless redirected with -o FILE.\n"
+           "\n"
+           "Errors are always dumped to stderr\n"
+           "\n"
+           "The '-f' and '-o' options may not be used with '-F', '--replace' or '--no-backup'.\n"
+           "The '--prefix' and '--suffix' options may not be used with '--replace' or '--no-backup'.\n"
+           "\n"
            "Basic Options:\n"
            " -c CFG       : use the config file CFG\n"
            " -f FILE      : process the single file FILE (output to stdout, use with -o)\n"
@@ -122,6 +144,9 @@ static void usage_exit(const char *msg, const char *argv0, int code)
            " --prefix PFX : Prepend PFX to the output filename path.\n"
            " --replace    : replace source files (creates a backup)\n"
            " --no-backup  : replace files, no backup. Useful if files are under source control\n"
+#ifdef HAVE_UTIME_H
+           " --mtime      : preserve mtime on replaced files\n"
+#endif
            " -l           : language override: C, CPP, D, CS, JAVA, PAWN\n"
            " -t           : load a file with types (usually not needed)\n"
            " -q           : quiet mode - no output on stderr (-L will override)\n"
@@ -140,10 +165,17 @@ static void usage_exit(const char *msg, const char *argv0, int code)
            " -s           : Show the log severity in the logs\n"
            " --decode FLAG: Print FLAG (chunk flags) as text and exit\n"
            "\n"
-           "If no input files are specified, the input is read from stdin\n"
-           "If -F is used or files are specified on the command line, the output is 'prefix/filename' + suffix\n"
-           "Otherwise, the output is dumped to stdout, unless redirected with -o FILE.\n"
-           "Errors are always dumped to stderr\n",
+           "Usage Examples\n"
+           "cat foo.d | uncrustify -q -c my.cfg -l d\n"
+           "uncrustify -c my.cfg -f foo.d\n"
+           "uncrustify -c my.cfg -f foo.d -L0-2,20-23,51\n"
+           "uncrustify -c my.cfg -f foo.d -o foo.d\n"
+           "uncrustify -c my.cfg foo.d\n"
+           "uncrustify -c my.cfg --replace foo.d\n"
+           "uncrustify -c my.cfg --no-backup foo.d\n"
+           "uncrustify -c my.cfg --prefix=out -F files.txt\n"
+           "\n"
+           ,
            path_basename(argv0));
    exit(code);
 }
@@ -154,6 +186,23 @@ static void version_exit(void)
    exit(0);
 }
 
+static void redir_stdout(const char *output_file)
+{
+   /* Reopen stdout */
+   FILE *my_stdout = stdout;
+   if (output_file != NULL)
+   {
+      my_stdout = freopen(output_file, "wb", stdout);
+      if (my_stdout == NULL)
+      {
+         LOG_FMT(LERR, "Unable to open %s for write: %s (%d)\n",
+                 output_file, strerror(errno), errno);
+         usage_exit(NULL, NULL, 56);
+      }
+      LOG_FMT(LNOTE, "Redirecting output to %s\n", output_file);
+   }
+}
+
 int main(int argc, char *argv[])
 {
    char       *data        = NULL;
@@ -224,11 +273,33 @@ int main(int argc, char *argv[])
    if (((cfg_file = arg.Param("--config")) == NULL) &&
        ((cfg_file = arg.Param("-c")) == NULL))
    {
-#ifdef WIN32
-      usage_exit("Specify the config file: -c file", argv[0], 58);
-#endif
+      /* Handled later */
    }
 
+#ifndef WIN32
+   /* Try to file a config at an alternate location */
+   char buf[512];
+   if (cfg_file == NULL)
+   {
+      cfg_file = getenv("UNCRUSTIFY_CONFIG");
+      if (cfg_file == NULL)
+      {
+         const char *home = getenv("HOME");
+
+         if (home != NULL)
+         {
+            struct stat tmp_stat;
+
+            snprintf(buf, sizeof(buf), "%s/.uncrustify.cfg", home);
+            if (stat(buf, &tmp_stat) == 0)
+            {
+               cfg_file = buf;
+            }
+         }
+      }
+   }
+#endif
+
    /* Get the parsed file name */
    if (((parsed_file = arg.Param("--parsed")) != NULL) ||
        ((parsed_file = arg.Param("-p")) != NULL))
@@ -287,25 +358,45 @@ int main(int argc, char *argv[])
    if (((source_file = arg.Param("--file")) == NULL) &&
        ((source_file = arg.Param("-f")) == NULL))
    {
-      // not using a single file
+      // not using a single file, source_file is NULL
    }
 
    if (((source_list = arg.Param("--files")) == NULL) &&
        ((source_list = arg.Param("-F")) == NULL))
    {
-      // not using a file list
+      // not using a file list, source_list is NULL
    }
 
    const char *prefix = arg.Param("--prefix");
    const char *suffix = arg.Param("--suffix");
 
-   bool no_backup = arg.Present("--no-backup");
-   if (arg.Present("--replace") || no_backup)
+   bool no_backup        = arg.Present("--no-backup");
+   bool replace          = arg.Present("--replace");
+   bool keep_mtime       = arg.Present("--mtime");
+   bool update_config    = arg.Present("--update-config");
+   bool update_config_wd = arg.Present("--update-config-with-doc");
+
+   /* Grab the output override */
+   output_file = arg.Param("-o");
+
+   LOG_FMT(LDATA, "output_file = %s\n", (output_file != NULL) ? output_file : "null");
+   LOG_FMT(LDATA, "source_file = %s\n", (source_file != NULL) ? source_file : "null");
+   LOG_FMT(LDATA, "source_list = %s\n", (source_list != NULL) ? source_list : "null");
+   LOG_FMT(LDATA, "prefix      = %s\n", (prefix != NULL) ? prefix : "null");
+   LOG_FMT(LDATA, "suffix      = %s\n", (suffix != NULL) ? suffix : "null");
+   LOG_FMT(LDATA, "replace     = %d\n", replace);
+   LOG_FMT(LDATA, "no_backup   = %d\n", no_backup);
+
+   if (replace || no_backup)
    {
       if ((prefix != NULL) || (suffix != NULL))
       {
          usage_exit("Cannot use --replace with --prefix or --suffix", argv[0], 66);
       }
+      if ((source_file != NULL) || (output_file != NULL))
+      {
+         usage_exit("Cannot use --replace or --no-backup with -f or -o", argv[0], 66);
+      }
    }
    else
    {
@@ -315,11 +406,17 @@ int main(int argc, char *argv[])
       }
    }
 
-   /* Grab the output override */
-   output_file = arg.Param("-o");
-
-   bool update_config    = arg.Present("--update-config");
-   bool update_config_wd = arg.Present("--update-config-with-doc");
+   /* Try to load the config file, if available.
+    * It is optional for "--universalindent", but required for everything else.
+    */
+   if (cfg_file != NULL)
+   {
+      cpd.filename = cfg_file;
+      if (load_option_file(cfg_file) < 0)
+      {
+         usage_exit("Unable to load the config file", argv[0], 56);
+      }
+   }
 
    if (arg.Present("--universalindent"))
    {
@@ -341,10 +438,30 @@ int main(int argc, char *argv[])
       return(EXIT_SUCCESS);
    }
 
+   /* Everything beyond this point requires a config file, so complain and
+    * bail if we don't have one.
+    */
+   if (cfg_file == NULL)
+   {
+#ifdef WIN32
+      usage_exit("Specify the config file: -c file", argv[0], 58);
+#else
+      usage_exit("Specify the config file with '-c file' or set UNCRUSTIFY_CONFIG",
+                 argv[0], 58);
+#endif
+   }
+
    /*
     *  Done parsing args
     */
 
+   if (update_config || update_config_wd)
+   {
+      redir_stdout(output_file);
+      save_option_file(stdout, update_config_wd);
+      return(0);
+   }
+
    /* Check for unused args (ignore them) */
    idx   = 1;
    p_arg = arg.Unused(idx);
@@ -360,58 +477,11 @@ int main(int argc, char *argv[])
 
       if (output_file != NULL)
       {
-         printf("source_file = %s, p_arg = %s\n", source_file, p_arg);
          usage_exit("Cannot specify -o with a mulit-file option.",
                     argv[0], 68);
       }
    }
 
-#ifndef WIN32
-   char buf[512];
-   if (cfg_file == NULL)
-   {
-      cfg_file = getenv("UNCRUSTIFY_CONFIG");
-      if (cfg_file == NULL)
-      {
-         const char *home = getenv("HOME");
-
-         if (home == NULL)
-         {
-            usage_exit("Specify the config file with '-c file' or set UNCRUSTIFY_CONFIG",
-                       argv[0], 58);
-         }
-         snprintf(buf, sizeof(buf), "%s/.uncrustify.cfg", home);
-         cfg_file = buf;
-      }
-   }
-#endif
-
-   cpd.filename = cfg_file;
-   if (load_option_file(cfg_file) < 0)
-   {
-      usage_exit("Unable to load the config file", argv[0], 56);
-   }
-
-   /* Reopen stdout */
-   FILE *my_stdout = stdout;
-   if (output_file != NULL)
-   {
-      my_stdout = freopen(output_file, "wb", stdout);
-      if (my_stdout == NULL)
-      {
-         LOG_FMT(LERR, "Unable to open %s for write: %s (%d)\n",
-                 output_file, strerror(errno), errno);
-         usage_exit(NULL, argv[0], 56);
-      }
-      LOG_FMT(LNOTE, "Redirecting output to %s\n", output_file);
-   }
-
-   if (update_config || update_config_wd)
-   {
-      save_option_file(stdout, update_config_wd);
-      return(0);
-   }
-
    /* This relies on cpd.filename being the config file name */
    load_header_files();
 
@@ -423,6 +493,8 @@ int main(int argc, char *argv[])
          cpd.lang_flags = LANG_C;
       }
 
+      redir_stdout(output_file);
+
       data = read_stdin(data_len);
       if (data == NULL)
       {
@@ -443,8 +515,8 @@ int main(int argc, char *argv[])
    }
    else if (source_file != NULL)
    {
-      /* Doing a single file, output to stdout */
-      do_source_file(source_file, stdout, parsed_file, NULL, NULL, no_backup);
+      /* Doing a single file */
+      do_source_file(source_file, output_file, parsed_file, no_backup, false);
    }
    else
    {
@@ -462,12 +534,15 @@ int main(int argc, char *argv[])
       idx = 1;
       while ((p_arg = arg.Unused(idx)) != NULL)
       {
-         do_source_file(p_arg, NULL, NULL, prefix, suffix, no_backup);
+         char outbuf[1024];
+         do_source_file(p_arg,
+                        make_output_filename(outbuf, sizeof(outbuf), p_arg, prefix, suffix),
+                        NULL, no_backup, keep_mtime);
       }
 
       if (source_list != NULL)
       {
-         process_source_list(source_list, prefix, suffix, no_backup);
+         process_source_list(source_list, prefix, suffix, no_backup, keep_mtime);
       }
    }
 
@@ -479,7 +554,7 @@ int main(int argc, char *argv[])
 
 static void process_source_list(const char *source_list,
                                 const char *prefix, const char *suffix,
-                                bool no_backup)
+                                bool no_backup, bool keep_mtime)
 {
    FILE *p_file = fopen(source_list, "r");
 
@@ -511,7 +586,10 @@ static void process_source_list(const char *source_list,
 
       if ((argc == 1) && (*args[0] != '#'))
       {
-         do_source_file(args[0], NULL, NULL, prefix, suffix, no_backup);
+         char outbuf[1024];
+         do_source_file(args[0],
+                        make_output_filename(outbuf, sizeof(outbuf), args[0], prefix, suffix),
+                        NULL, no_backup, keep_mtime);
       }
    }
 }
@@ -559,10 +637,13 @@ static char *read_stdin(int& out_len)
    return(data);
 }
 
-static void make_folders(char *outname)
+static void make_folders(const char *filename)
 {
    int idx;
    int last_idx = 0;
+   char outname[1024];
+
+   snprintf(outname, sizeof(outname), "%s", filename);
 
    for (idx = 0; outname[idx] != 0; idx++)
    {
@@ -603,6 +684,17 @@ int load_mem_file(const char *filename, file_mem& fm)
    fm.data   = NULL;
    fm.length = 0;
 
+   /* Grab the stat info for the file */
+   if (stat(filename, &my_stat) < 0)
+   {
+      return(-1);
+   }
+
+#ifdef HAVE_UTIME_H
+   /* Save off mtime */
+   fm.utb.modtime = my_stat.st_mtime;
+#endif
+
    /* Try to read in the file */
    p_file = fopen(filename, "rb");
    if (p_file == NULL)
@@ -610,9 +702,6 @@ int load_mem_file(const char *filename, file_mem& fm)
       return(-1);
    }
 
-   /* this really can't fail... */
-   (void)fstat(fileno(p_file), &my_stat);
-
    fm.length = my_stat.st_size;
    fm.data   = (char *)malloc(fm.length + 1);
    if (fm.data == NULL)
@@ -642,7 +731,6 @@ int load_mem_file(const char *filename, file_mem& fm)
    return(retval);
 }
 
-
 /**
  * Try to load the file from the config folder first and then by name
  */
@@ -667,7 +755,6 @@ static int load_mem_file_config(const char *filename, file_mem& fm)
    return(retval);
 }
 
-
 static int load_header_files()
 {
    int retval = 0;
@@ -693,78 +780,96 @@ static int load_header_files()
    return(retval);
 }
 
+static const char *make_output_filename(char *buf, int buf_size,
+                                        const char *filename,
+                                        const char *prefix,
+                                        const char *suffix)
+{
+   int len = 0;
+
+   if (prefix != NULL)
+   {
+      len = snprintf(buf, buf_size, "%s/", prefix);
+   }
+
+   snprintf(&buf[len], buf_size - len, "%s%s", filename,
+            (suffix != NULL) ? suffix : "");
+
+   return(buf);
+}
+
+
 /**
  * Does a source file.
- * If pfout is NULL, the source fileaname + ".uncrustify" is used.
  *
- * @param filename the file to read
- * @param pfout    NULL or the output stream
+ * @param filename_in  the file to read
+ * @param filename_out NULL (stdout) or the file to write
+ * @param parsed_file  NULL or the filename for the parsed debug info
+ * @param no_backup    don't create a backup, if filename_out == filename_in
+ * @param keep_mtime   don't change the mtime (dangerous)
  */
-static void do_source_file(const char *filename, FILE *pfout, const char *parsed_file,
-                           const char *prefix, const char *suffix, bool no_backup)
+static void do_source_file(const char *filename_in,
+                           const char *filename_out,
+                           const char *parsed_file,
+                           bool no_backup,
+                           bool keep_mtime)
 {
+   FILE     *pfout;
    bool     did_open    = false;
    bool     need_backup = false;
    file_mem fm;
 
-   /* Do some simple language detection based on the filename */
+   /* Do some simple language detection based on the filename extension */
    if (cpd.lang_flags == 0)
    {
-      cpd.lang_flags = language_from_filename(filename);
+      cpd.lang_flags = language_from_filename(filename_in);
    }
 
    /* Try to read in the source file */
-   if (load_mem_file(filename, fm) < 0)
+   if (load_mem_file(filename_in, fm) < 0)
    {
-      LOG_FMT(LERR, "Failed to load (%s)\n", filename);
+      LOG_FMT(LERR, "Failed to load (%s)\n", filename_in);
       cpd.error_count++;
       return;
    }
 
    LOG_FMT(LSYS, "Parsing: %s as language %s\n",
-           filename, language_to_string(cpd.lang_flags));
+           filename_in, language_to_string(cpd.lang_flags));
 
-   if (pfout == NULL)
+   if (filename_out == NULL)
    {
-      char outname[1024];
-      int  len = 0;
-
-      if (!no_backup && (prefix == NULL) && (suffix == NULL))
+      pfout = stdout;
+   }
+   else
+   {
+      if (!no_backup && (strcmp(filename_in, filename_out) == 0))
       {
-         if (backup_copy_file(filename, fm.data, fm.length) != SUCCESS)
+         if (backup_copy_file(filename_in, fm.data, fm.length) != SUCCESS)
          {
             LOG_FMT(LERR, "%s: Failed to create backup file for %s\n",
-                    __func__, filename);
+                    __func__, filename_in);
             free(fm.data);
+            cpd.error_count++;
             return;
          }
          need_backup = true;
       }
+      make_folders(filename_out);
 
-      if (prefix != NULL)
-      {
-         len = snprintf(outname, sizeof(outname), "%s/", prefix);
-      }
-
-      snprintf(&outname[len], sizeof(outname) - len, "%s%s", filename,
-               (suffix != NULL) ? suffix : "");
-
-      make_folders(outname);
-
-      pfout = fopen(outname, "wb");
+      pfout = fopen(filename_out, "wb");
       if (pfout == NULL)
       {
          LOG_FMT(LERR, "%s: Unable to create %s: %s (%d)\n",
-                 __func__, outname, strerror(errno), errno);
+                 __func__, filename_out, strerror(errno), errno);
          cpd.error_count++;
          free(fm.data);
          return;
       }
       did_open = true;
-      //LOG_FMT(LSYS, "Output file %s\n", outname);
+      //LOG_FMT(LSYS, "Output file %s\n", filename_out);
    }
 
-   cpd.filename = filename;
+   cpd.filename = filename_in;
    uncrustify_file(fm.data, fm.length, pfout, parsed_file);
 
    free(fm.data);
@@ -773,10 +878,19 @@ static void do_source_file(const char *filename, FILE *pfout, const char *parsed
    {
       fclose(pfout);
 
-      if (need_backup && !no_backup)
+      if (need_backup)
       {
-         backup_create_md5_file(filename);
+         backup_create_md5_file(filename_in);
       }
+
+#ifdef HAVE_UTIME_H
+      if (keep_mtime)
+      {
+         /* update mtime -- don't care if it fails */
+         fm.utb.actime = time(NULL);
+         (void)utime(filename_in, &fm.utb);
+      }
+#endif
    }
 }
 
@@ -880,6 +994,12 @@ static void uncrustify_file(const char *data, int data_len, FILE *pfout,
       add_func_header(CT_CLASS, cpd.class_hdr);
    }
 
+   /* Scrub extra semicolons */
+   if (cpd.settings[UO_mod_remove_extra_semicolon].b)
+   {
+      remove_extra_semicolons();
+   }
+
    /**
     * Look at all colons ':' and mark labels, :? sequences, etc.
     */
@@ -938,10 +1058,19 @@ static void uncrustify_file(const char *data, int data_len, FILE *pfout,
       pawn_scrub_vsemi();
    }
 
-   /* Scrub extra semicolons */
-   if (cpd.settings[UO_mod_remove_extra_semicolon].b)
+   /* Insert trailing comments after certain close braces */
+   if ((cpd.settings[UO_mod_add_long_switch_closebrace_comment].n > 0) ||
+       (cpd.settings[UO_mod_add_long_function_closebrace_comment].n > 0))
    {
-      remove_extra_semicolons();
+      add_long_closebrace_comment();
+   }
+
+   /* Sort imports/using/include */
+   if (cpd.settings[UO_mod_sort_import].b ||
+       cpd.settings[UO_mod_sort_include].b ||
+       cpd.settings[UO_mod_sort_using].b)
+   {
+      sort_imports();
    }
 
    /**
@@ -979,6 +1108,7 @@ static void uncrustify_file(const char *data, int data_len, FILE *pfout,
          do_code_width();
          if (prev_changes != cpd.changes)
          {
+            align_all();
             indent_text();
          }
       } while ((prev_changes != cpd.changes) && (--max_passes > 0));
@@ -1048,24 +1178,25 @@ struct file_lang
 
 struct file_lang languages[] =
 {
-   { ".c",    "C",    LANG_C    },
-   { ".cpp",  "CPP",  LANG_CPP  },
-   { ".d",    "D",    LANG_D    },
-   { ".cs",   "CS",   LANG_CS   },
-   { ".vala", "VALA", LANG_VALA },
-   { ".java", "JAVA", LANG_JAVA },
-   { ".pawn", "PAWN", LANG_PAWN },
-   { ".p",    "",     LANG_PAWN },
-   { ".sma",  "",     LANG_PAWN },
-   { ".inl",  "",     LANG_PAWN },
-   { ".h",    "",     LANG_CPP  },
-   { ".cxx",  "",     LANG_CPP  },
-   { ".hpp",  "",     LANG_CPP  },
-   { ".hxx",  "",     LANG_CPP  },
-   { ".cc",   "",     LANG_CPP  },
-   { ".di",   "",     LANG_D    },
-   { ".m",    "OC",   LANG_OC   },
-   { ".sqc",  "",     LANG_C    }, // embedded SQL
+   { ".c",    "C",    LANG_C             },
+   { ".cpp",  "CPP",  LANG_CPP           },
+   { ".d",    "D",    LANG_D             },
+   { ".cs",   "CS",   LANG_CS            },
+   { ".vala", "VALA", LANG_VALA          },
+   { ".java", "JAVA", LANG_JAVA          },
+   { ".pawn", "PAWN", LANG_PAWN          },
+   { ".p",    "",     LANG_PAWN          },
+   { ".sma",  "",     LANG_PAWN          },
+   { ".inl",  "",     LANG_PAWN          },
+   { ".h",    "",     LANG_CPP           },
+   { ".cxx",  "",     LANG_CPP           },
+   { ".hpp",  "",     LANG_CPP           },
+   { ".hxx",  "",     LANG_CPP           },
+   { ".cc",   "",     LANG_CPP           },
+   { ".di",   "",     LANG_D             },
+   { ".m",    "OC",   LANG_OC            },
+   { ".mm",   "OC+",  LANG_OC | LANG_CPP },
+   { ".sqc",  "",     LANG_C             }, // embedded SQL
 };
 
 /**
@@ -1135,6 +1266,16 @@ static const char *language_to_string(int lang)
 {
    int i;
 
+   /* Check for an exact match first */
+   for (i = 0; i < (int)ARRAY_SIZE(languages); i++)
+   {
+      if (languages[i].lang == lang)
+      {
+         return(languages[i].tag);
+      }
+   }
+
+   /* Check for the first set language bit */
    for (i = 0; i < (int)ARRAY_SIZE(languages); i++)
    {
       if ((languages[i].lang & lang) != 0)
