@@ -40,5 +40,13 @@ void foo(void)
    {
       ;
    }
+
+   foobar(param1,
+          param2, param3,
+          param4);
+
+   foobar2(param1
+           , param2, param3
+           , param4);
 }
 
