@@ -68,3 +68,14 @@ void f()
                  disks_props.push_back (entry);
              }
 }
+
+template < class > struct type;
+
+template < class T > 
+class X {
+  typedef type < T > base ;
+  void f () {
+    ( base :: operator * () ) ;; 
+  };
+};
+
