@@ -8,8 +8,8 @@ void foo(void)
       a++;
    }
 
-   while (b < a)
-   {              // trailing C++ comment
+   while (b < a)  // trailing C++ comment
+   {
       b++;
    }
 
@@ -50,8 +50,8 @@ void foo(void)
          {
             a /= b;
          }
-         else
-         {    // trailing C++ comment
+         else // trailing C++ comment
+         {
             a += b;
          }
       }
@@ -71,8 +71,8 @@ void foo(void)
 
    while (1)
    {
-      ;
-   }     /* hang forever */
+      ;  /* hang forever */
+   }
 }
 
 void f()
@@ -91,3 +91,11 @@ void f()
       buf = mallocstrcpy(buf, tmp);
    }
 }
+
+void f()
+{
+}
+void g()
+{
+}
+
