@@ -11,7 +11,7 @@ void foo(void)
 #ifdef FLAG2
        || flag2
 #endif
-   )
+       )
    {
       printf("yar");
    }
@@ -40,5 +40,13 @@ void foo(void)
    {
       ;
    }
+
+   foobar(param1,
+          param2, param3,
+          param4);
+
+   foobar2(param1,
+           param2, param3,
+           param4);
 }
 
