@@ -29,7 +29,7 @@ void asd(void)
    {
       a = b < c > 0;
    }
-   if (a < bar() > c)
+   if (a<bar()>c)
    {
    }
    a < up_lim() ? do_hi() : do_low;
@@ -121,3 +121,25 @@ struct foo {
    type1<int const>bar;
 };
 
+
+template<int i>void f();
+template<int i>void g() {
+   f<i - 1>();
+   f<i>();
+   f<i + 1>();
+   f<bar()>();
+}
+void h() {
+   g<42>();
+}
+
+#include <vector>
+std::vector<int>A(2);
+std::vector<int>B;
+std::vector<int>C(2);
+std::vector<int>D;
+
+template<class T>struct X {template<class U>void operator()(U);};
+
+template<class T>class Y {template<class V>void f(V);};
+
