@@ -1,37 +1,44 @@
+int &
+Function()
+{
+   static int x;
+   return (x);
+}
+
 void
 foo1(
    int param1,
    int param2,
    char *param2
-);
+   );
 
 void
 foo2(
    int param1,
    int param2,
    char *param2
-);
+   );
 
 void
 foo3(
    int param1,
    int param2,                                               // comment
    char *param2
-);
+   );
 
 struct whoopee *
 foo4(
    int param1,
    int param2,
    char *param2                                              /* comment */
-);
+   );
 
 const struct snickers *
 foo5(
    int param1,
    int param2,
    char *param2
-);
+   );
 
 
 void
@@ -39,14 +46,16 @@ foo(
    int param1,
    int param2,
    char *param2
-)
+   )
 {
    printf("boo!\n");
 }
 
-int classname::method();
+int
+classname::method();
 
-int classname::method()
+int
+classname::method()
 {
    foo();
 }
@@ -63,7 +72,7 @@ classname::method2()
 const int&
 className::method1(
    void
-) const
+   ) const
 {
    // stuff
 }
@@ -71,8 +80,20 @@ className::method1(
 const longtypename&
 className::method2(
    void
-) const
+   ) const
 {
    // stuff
 }
 
+int &
+foo();
+
+int &
+foo()
+{
+   list_for_each(a,b) {
+      bar(a);
+   }
+   return nuts;
+}
+
