@@ -5,24 +5,19 @@ Function()
    return (x);
 }
 
-void
-foo1(int param1, int param2, char *param2);
+void foo1(int param1, int param2, char *param2);
 
-void
-foo2(int param1,
-     int param2,
-     char *param2);
+void foo2(int param1,
+          int param2,
+          char *param2);
 
-void
-foo3(int param1,
-     int param2,                                             // comment
-     char *param2);
+void foo3(int param1,
+          int param2,                                        // comment
+          char *param2);
 
-struct whoopee *
-foo4(int param1, int param2, char *param2 /* comment */);
+struct whoopee *foo4(int param1, int param2, char *param2 /* comment */);
 
-const struct snickers *
-foo5(int param1, int param2, char *param2);
+const struct snickers *foo5(int param1, int param2, char *param2);
 
 
 void
@@ -31,8 +26,7 @@ foo(int param1, int param2, char *param2)
    printf("boo!\n");
 }
 
-int
-classname::method();
+int classname::method();
 
 int
 classname::method()
@@ -40,8 +34,7 @@ classname::method()
    foo();
 }
 
-int
-classname::method2();
+int classname::method2();
 
 int
 classname::method2()
@@ -61,8 +54,7 @@ className::method2(void) const
    // stuff
 }
 
-int &
-foo();
+int &foo();
 
 int &
 foo()
