@@ -1,3 +1,9 @@
+int &Function()
+{
+   static int x;
+   return (x);
+}
+
 void foo1(int param1, int param2, char *param2);
 
 void foo2(int param1,int param2,char *param2);
@@ -22,11 +28,9 @@ int classname::method()
    foo();
 }
 
-int
-classname::method2();
+int classname::method2();
 
-int
-classname::method2()
+int classname::method2()
 {
    foo2();
 }
@@ -41,3 +45,13 @@ const longtypename& className::method2(void) const
    // stuff
 }
 
+int &foo();
+
+int &foo()
+{
+   list_for_each(a,b) {
+      bar(a);
+   }
+   return nuts;
+}
+
