@@ -45,7 +45,6 @@ enum FPP {
 struct narg {
 	int abc;
 	char def;
-
 	const char *ghi;
 };
 
@@ -71,3 +70,14 @@ void f()
 		disks_props.push_back (entry);
 	}
 }
+
+template < class > struct type;
+
+template < class T >
+class X {
+typedef type < T > base ;
+void f () {
+	( base :: operator * () ) ;
+}
+};
+
